unit graphica_guibase;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fgl, gl, graphica_vector, graphica_globject,
  graphica_gldriveinfo, graphica_glshader, graphica_glrenderinfo;

type
  TGGUISize = record
    left,top,width,height:Single;
  end;
  PGGUISize=^TGGUISize;
  TGGUISizeConstrant = record
    minWidth,minHeight,maxWidth,maxHeight:Single;
  end;
//  PGGUISizeConstrant=^TGGUISizeConstrant;

  TGGUIEvent=(gbeNone,gbeDown,gbeUp,gbeMove,gbeEnter,gbeLeave,
           gbeWheel,gbeWheelUp,gbeWheelDown,gbeKeys);
  TGGUIEvents = set of TGGUIEvent;

  TGGUIBase = class;
  TGGUIRenderParams = object
    GLControlInfo:TGGLControlInfo;
    GUIObject:TGGUIBase;
    data:Pointer;
  end;
  PGGUIRenderParams=^TGGUIRenderParams;

  TGLRenderParamsList = class(specialize TFPGMap <Pointer, PGGUIRenderParams>);
  TGLRenderParamsMap = class
  private
     FRenderParamsList:TGLRenderParamsList;
     FGUIObject:TGGUIBase;
  public
     property RenderParamsList:TGLRenderParamsList read FRenderParamsList;
  end;
  PGLRenderParamsMap=^TGLRenderParamsMap;


  TGGUIPerformance=(gpNone,gpSelect,gpClick,gpDbClick,gpMouseEvents,gpMove);
  TGGUIPerformances = set of TGGUIPerformance;


  { TGGUIBase }

  TGGUIBase = class(TGGLObject)
  private
  protected
    FConstraints:TGGUISizeConstrant;
    FEvents:TGGUIEvents;
    FRenderParams:PGGUIRenderParams;
    FAlpha:Single;
    FPerformances:TGGUIPerformances;

    function getMeasure():TGGUISize;virtual;abstract;
    procedure paintGUI();dynamic;abstract;
    procedure InitObject;override;
    procedure DoRenderGUI(const renderinfo:TGRenderInfo;
      pRenderParams:PGGUIRenderParams);dynamic;abstract;
    procedure DoRender(const renderinfo:TGRenderInfo);override;
    procedure internalMouseEvent(e:TGGUIEvent;v:PGVector3f);dynamic;
    procedure internalJoinRenderParams(GLControlInfo:TGGLControlInfo);dynamic;abstract;
//    procedure ClearRenderParamsMap;
    procedure destroyRenderParams;virtual;
  public
    constructor Create(own:TGGLObject);
    constructor Create(own:TGGUIBase);
    destructor Destroy;override;
    procedure JoinRenderParams(GLControlInfo:TGGLControlInfo);
    procedure SetMouseEvent(e:TGGUIEvent;v:PGVector3f);
    procedure Paint();virtual;abstract;

    property Performances:TGGUIPerformances read FPerformances;
    property RenderParams:PGGUIRenderParams read FRenderParams;
  end;

implementation
uses glext,graphica_glplane, Unix;
{ TGGUIBase }

procedure TGGUIBase.InitObject;
begin
  inherited InitObject;
  FRenderParams:=nil;
//  FRenderParamsMap:=TGLRenderParamsMap.Create;
//  FRenderParams:=TGGUIRenderParams;
end;

procedure TGGUIBase.DoRender(const renderinfo: TGRenderInfo);
begin
  if FRenderParams=nil then exit;
  if not FVisible then exit;
  if renderinfo.SelectMode then begin
     if not Selected then exit;
  //  rp:=FRenderParamsMap.KeyData[renderinfo.GLControlInfo.GLControl];
//    if(rp<>nil) then
    renderinfo.SelectDriver.Start(renderinfo.View.GLMVP,TAMM,FSizeMatrix);
    gl_draw_panel_list(renderinfo.GLQuardListID);
    renderinfo.SelectDriver.Stop(Self,renderinfo.View);
  end else begin
    renderinfo.Style.Shader.Active();
    glUniformMatrix4fv(renderinfo.Style.ShaderParams.MVP, 1, GL_FALSE, PGLfloat(@renderinfo.View.GLMVP));
    glUniformMatrix4fv(renderinfo.Style.ShaderParams.MMA, 1, GL_FALSE, @FTAMM);
    glUniformMatrix4fv(renderinfo.Style.ShaderParams.MMS, 1, GL_FALSE, @FSizeMatrix);
    glUniform1i(renderinfo.Style.ShaderParams.Texture,0);
    glUniform1f(renderinfo.Style.ShaderParams.Alpha, FAlpha);
    DoRenderGUI(renderinfo,FRenderParams);
    renderinfo.Style.Shader.Deactive();
  end;
end;

procedure TGGUIBase.internalMouseEvent(e: TGGUIEvent; v: PGVector3f);
begin

end;

procedure TGGUIBase.destroyRenderParams;
begin
  Dispose(FRenderParams);
  FRenderParams:=nil;
end;

{procedure TGGUIBase.ClearRenderParamsMap;
var
  i:Integer;
begin
//  for i:=0 to FRenderParamsMap.Count-1 do
//    Dispose(FRenderParamsMap.Data[i]);
//  FRenderParamsMap.Clear;
end;}

constructor TGGUIBase.Create(own: TGGLObject);
begin
  inherited Create(own);
end;

constructor TGGUIBase.Create(own: TGGUIBase);
begin
  inherited Create(own);
end;

destructor TGGUIBase.Destroy;
begin
  destroyRenderParams;
//  ClearRenderParamsMap;
//  FreeAndNil(FRenderParamsMap);
  inherited Destroy;
end;

procedure TGGUIBase.JoinRenderParams(GLControlInfo: TGGLControlInfo);
begin
  internalJoinRenderParams(GLControlInfo);
end;

procedure TGGUIBase.SetMouseEvent(e: TGGUIEvent; v: PGVector3f);
begin
  internalMouseEvent(e,v);
end;

end.

