unit graphica_guidrive;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Controls, syncobjs, gl, OpenGLContext,
  graphica_globject,graphica_glrenderinfo,opengraphicscontrol,
  graphica_glselect, graphica_guibase, graphica_vector;

type

  TGGUIMouseEvent = (gmeNone,gmeMove,gmeDown,gmeUp);


  TGGUIMouseInfo = record
    x,y:Integer;
    event:TGGUIMouseEvent;
    button: TMouseButton;
    shift: TShiftState;
    lock:TCriticalSection;
  end;

  TGGUIDriveState = (gdsNone,gdsMoveForm);

  TGGUISelectInfo = record
    obj:TGGUIBase;
    offsetPos:TGVector3f;
    startNormal,startPos:TGVector3f;
  end;


  { TGGUIDrive }

  TGGUIDrive = class
  private
    FState:TGGUIDriveState;
    FMouseInfo:TGGUIMouseInfo;
    FRootObjects:TGGLObject;
    FRenderInfo:TGRenderInfo;
    FLock:TCriticalSection;
    FSelectInfo:TGGUISelectInfo;
  public
    constructor create(const ro:TGGLObject;const w:TGOpenGLControl;
      QuardListID:GLuint);
    destructor destroy();override;

    procedure mousemove(Shift: TShiftState; X, Y: Integer);
    procedure mousedown(Button: TMouseButton;Shift: TShiftState; X, Y: Integer);
    procedure mouseup(Button: TMouseButton;Shift: TShiftState; X, Y: Integer);


    procedure select();
    procedure render();

    property RootObjects:TGGLObject read FRootObjects;
    property RenderInfo:TGRenderInfo read FRenderInfo;
  end;

implementation
uses graphica_2d3d, graphica_glview, graphica_guivisualcomponents, LazLogger;

{ TGGUIDrive }

constructor TGGUIDrive.create(const ro: TGGLObject; const w: TGOpenGLControl;
  QuardListID: GLuint);
begin
  FState:=gdsNone;
  FMouseInfo.event:=gmeNone;
  FRenderInfo:=TGRenderInfo.Create(w,QuardListID);
  FRootObjects:=ro;
  FLock:=TCriticalSection.Create;
  FSelectInfo.obj:=nil;
end;

destructor TGGUIDrive.destroy;
begin
  FreeAndNil(FLock);
  FreeAndNil(FRenderInfo);
  inherited destroy;
end;

procedure TGGUIDrive.mousemove(Shift: TShiftState; X, Y: Integer);
begin
  FLock.Enter;
  try
    FMouseInfo.event:=gmeMove;
    FMouseInfo.x:=X;
    FMouseInfo.y:=Y;
    FMouseInfo.Shift:=[];
  finally
    FLock.Leave;
  end;
end;

procedure TGGUIDrive.mousedown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  FLock.Enter;
  try
    FMouseInfo.event:=gmeDown;
    FMouseInfo.x:=X;
    FMouseInfo.y:=Y;
    FMouseInfo.Button:=Button;
    FMouseInfo.Shift:=Shift;
  finally
    FLock.Leave;
  end;
end;

procedure TGGUIDrive.mouseup(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  FLock.Enter;
  try
    FMouseInfo.event:=gmeUp;
    FMouseInfo.x:=X;
    FMouseInfo.y:=Y;
    FMouseInfo.Button:=Button;
    FMouseInfo.Shift:=Shift;
    if FState in [gdsMoveForm] then
       FState:=gdsNone;
  finally
    FLock.Leave;
  end;
end;

procedure TGGUIDrive.select;
var
  selectCL:TGLSelectColorList;
  guiObj:TGGUIBase;
  viewRay:TGGLViewRay;
  v3f:TGVector3f;
begin
  FLock.Enter;
  try
    if(FMouseInfo.event<>gmeNone)then begin
      glDisable(GL_TEXTURE_2D);
      glDepthFunc(GL_ALWAYS);
      FRenderInfo.SelectMode:=true;
      FRenderInfo.SelectDriver.setParameters(FMouseInfo.x,FMouseInfo.y);
      FRootObjects.Render(FRenderInfo);
      if FState<>gdsMoveForm then begin
        selectCL:=(FRenderInfo.SelectDriver as TGLSelectColorList);
        if selectCL<>nil then begin
          if selectCL.Count>0 then begin
            guiObj:=selectCL.getObject(selectCL.Count-1) as TGGUIBase;
            case FMouseInfo.event of
              gmeMove: begin
                 if(gpMove in guiObj.Performances) then begin
                   if (FState = gdsNone) then begin
                   end;
                   if (FState = gdsMoveForm) then begin
                     if(guiObj is TGGUIForm) then begin
                        viewRay:=RenderInfo.View.getViewRay(FMouseInfo.x,FMouseInfo.y);
                     end;
                   end;
                 end;
              end;
              gmeDown: begin
                 if(gpSelect in guiObj.Performances) then begin
                   if (FState = gdsNone) then begin
                     if(guiObj is TGGUIForm) then begin
                        viewRay:=RenderInfo.View.getViewRay(FMouseInfo.x,FMouseInfo.y);
                        if(intersectRayToPlane(v3f,guiObj.GetPosition(),guiObj.GetNormal(),
                                 viewRay.pointZNear,viewRay.normal)) then begin
                                    FSelectInfo.obj:=guiObj;
                                    FSelectInfo.offsetPos:=guiObj.GetPosition()-v3f;
                                    FSelectInfo.startNormal:=guiObj.GetNormal();
                                    FSelectInfo.startPos:=guiObj.GetPosition();
                                    FState:=gdsMoveForm;
                                  end;
                     end;
                   end;
                 end;
              end;
            end;
          end else begin // if selectCL.Count>0 then begin


          end;
        end; //if selectCL<>nil then


      end else begin //if FState<>gdsMoveForm then begin
         if FState=gdsMoveForm then begin
           case FMouseInfo.event of
             gmeMove: begin
                viewRay:=RenderInfo.View.getViewRay(FMouseInfo.x,FMouseInfo.y);
                if(intersectRayToPlane(v3f,FSelectInfo.startPos,FSelectInfo.startNormal,
                          viewRay.pointZNear,viewRay.normal)) then begin
                            DebugLn('x:'+IntToStr(FMouseInfo.x)+', y:'+IntToStr(FMouseInfo.y)+
                            '  x:'+FloatToStr(v3f[0])+' ,y:'+FloatToStr(v3f[1]));
                            v3f:=v3f+FSelectInfo.offsetPos;
                            FSelectInfo.obj.SetPosition(v3f);
                          end;
             end;
           end;
         end;
      end;
      FRenderInfo.SelectDriver.Clear;
      FMouseInfo.event:=gmeNone;
    end;//if(FMouseInfo.event<>gmeNone)then begin
  finally
    FLock.Leave;
  end;
end;

procedure TGGUIDrive.render;
begin
  glEnable(GL_TEXTURE_2D);
  glDepthFunc(GL_LEQUAL);
  FRenderInfo.SelectMode:=false;
  FRootObjects.Render(FRenderInfo);
end;

end.

