unit graphica_guivisualcomponents;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, gl, graphica_vector, graphica_guibase,
  graphica_gltexture, graphica_glshader, graphica_glrenderinfo,
  graphica_gldriveinfo;


type

  TGGUIOnMouseMove = procedure(const self:TGGUIBase;constref v:TGVector3f) of object;
  TGGUIOnMouseUpDown = procedure(const self:TGGUIBase;constref v:TGVector3f) of object;

  TGGUIRenderParamsPlane = object(TGGUIRenderParams)
    Ftexture2D:TGGLTexture;
  end;
  PGGUIRenderParamsPlane=^TGGUIRenderParamsPlane;


  { TGGUIPlane }

  TGGUIPlane = class(TGGUIBase)
  private
//    FMVP,FMA,FMMS,FTextureID,FAlphaID:GLint;
//    Ftexture2D:TGGLTexture;
//    FRenderParamsPlane:TGGUIRenderParamsPlane;
    FColor:TGVector4f;

    FOnMouseMove:TGGUIOnMouseMove;
    FOnMouseUp:TGGUIOnMouseUpDown;
    FOnMouseDown:TGGUIOnMouseUpDown;
  protected
    function getMeasure():TGGUISize;override;
    procedure paintGUI();override;
    procedure DoRenderGUI(const renderinfo:TGRenderInfo;
      pRenderParams:PGGUIRenderParams);override;
    procedure FinishRender(const renderinfo:TGRenderInfo);override;
    procedure InitObject;override;
    procedure internalMouseEvent(e:TGGUIEvent;v:PGVector3f);override;
    procedure internalJoinRenderParams(GLControlInfo:TGGLControlInfo);override;
    procedure destroyRenderParams;override;
  public
    destructor Destroy; override;
    procedure Paint();override;

    property OnMouseMove:TGGUIOnMouseMove read FOnMouseMove
                                                 write FOnMouseMove;
    property OnMouseDown:TGGUIOnMouseUpDown read FOnMouseDown
                                                 write FOnMouseDown;
    property OnMouseUp:TGGUIOnMouseUpDown read FOnMouseUp
                                                 write FOnMouseUp;
    property Constraints:TGGUISizeConstrant read FConstraints
                                                 write FConstraints;
    property Alpha:Single read FAlpha write FAlpha;
  end;


  { TGGUIForm }

  TGGUIForm = class(TGGUIPlane)
  private
    FCaption:String;
  protected
    procedure InitObject;override;
    procedure paintGUI();override;
    procedure SetCaption(v:String);
  public
    property Caption:String read FCaption write SetCaption;
  end;


  { TGGUILabel }

  TGGUILabel = class(TGGUIPlane)
  private

  protected
    procedure InitObject;override;
  end;


  procedure InitGGUI();

implementation
uses types,IntfGraphics, GraphType, Graphics, GLext, FPimage,
  FPCanvas,FPImgCanv, ftfont, graphica_glplane;
//var
//  ___GGUIShader:TGGLShader;

{$i graphica_guivisualcomponents.inc}

procedure InitGGUI;
begin
//  ___GGUIShader:=TGGLShader.Create(V);
end;

{ TGGUILabel }

procedure TGGUILabel.InitObject;
begin
  inherited InitObject;
end;

{ TGGUIForm }

procedure TGGUIForm.InitObject;
begin
  inherited InitObject;
  FPerformances:=[gpSelect,gpClick,gpMove,gpMouseEvents];
end;

procedure TGGUIForm.paintGUI;
begin
  if FRenderParams<>nil then
     BeginThread(@threadPaintForm,FRenderParams);
end;

procedure TGGUIForm.SetCaption(v: String);
begin
  FCaption:=v;
  paintGUI();
end;

{ TGGUIPlane }

function TGGUIPlane.getMeasure: TGGUISize;
var
  sizePlane:TGVector3f;
begin
  sizePlane:=GetSize();
  Result.top:=0;
  Result.left:=0;
  Result.height:=sizePlane[0]-4;
  Result.width:=sizePlane[1]-4;
end;

procedure TGGUIPlane.paintGUI;
begin
  if FRenderParams<>nil then
     BeginThread(@threadPaintPlane,FRenderParams);
end;

procedure TGGUIPlane.DoRenderGUI(const renderinfo: TGRenderInfo;
  pRenderParams: PGGUIRenderParams);
var
  rp:PGGUIRenderParamsPlane;
begin
  rp:=PGGUIRenderParamsPlane(pRenderParams);
  glActiveTexture(GL_TEXTURE0);
  rp^.Ftexture2D.bind();
  glEnable(GL_STENCIL_TEST);
  glClear(GL_STENCIL_BUFFER_BIT);
  glStencilMask($FF);
  glStencilFunc(GL_ALWAYS, 1, $FF);
  glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
  gl_draw_panel_list(renderinfo.GLQuardListID);
  rp^.Ftexture2D.unBind();
end;

procedure TGGUIPlane.FinishRender(const renderinfo: TGRenderInfo);
begin
  if(renderinfo.GLStencilMask=0) then exit;
  renderinfo.GLStencilMask:=0;
  glDisable(GL_STENCIL_TEST);
end;


procedure TGGUIPlane.InitObject;
begin
  inherited InitObject;
  FAlpha:=0.5;
  FEvents:=[];
end;

procedure TGGUIPlane.internalMouseEvent(e: TGGUIEvent; v: PGVector3f);
begin
  inherited internalMouseEvent(e, v);
end;

{procedure TGGUIPlane.internalJoinRenderParams(GLControlInfo: TGGLControlInfo);
var
  i:Integer;
  pr:PGGUIRenderParamsPlane;
begin
  i:=FRenderParamsMap.IndexOf(GLControlInfo.GLControl);
  if (i<0) then begin
     New(pr);
     pr^.GLControlInfo:=GLControlInfo;
     pr^.GUIObject:=Self;
     GLControlInfo.Lock.Enter;
     try
       GLControlInfo.GLControl.MakeCurrent();
       pr^.Ftexture2D:=TGGLTexture.Create();
       pr^.Ftexture2D.bind();
       pr^.Ftexture2D.parameter(GL_NEAREST,GL_NEAREST,GL_REPEAT,GL_REPEAT,GL_REPLACE);
       pr^.Ftexture2D.unBind();
       FRenderParamsMap.AddOrSetData(GLControlInfo.GLControl,pr);
       GLControlInfo.GLControl.ReleaseContext();
     finally
       GLControlInfo.Lock.Leave;
     end;
  end;
end;}

procedure TGGUIPlane.internalJoinRenderParams(GLControlInfo: TGGLControlInfo);
var
  pr:PGGUIRenderParamsPlane;
begin
  if FRenderParams=nil then begin
     New(pr);
     pr^.GLControlInfo:=GLControlInfo;
     pr^.GUIObject:=TGGUIBase(Self);
     GLControlInfo.Lock.Enter;
     try
       GLControlInfo.GLControl.MakeCurrent();
       pr^.Ftexture2D:=TGGLTexture.Create();
       pr^.Ftexture2D.bind();
       pr^.Ftexture2D.parameter(GL_NEAREST,GL_NEAREST,GL_REPEAT,GL_REPEAT,GL_REPLACE);
       pr^.Ftexture2D.unBind();
       GLControlInfo.GLControl.ReleaseContext();
       FRenderParams:=pr;
     finally
       GLControlInfo.Lock.Leave;
     end;
  end;
end;

procedure TGGUIPlane.destroyRenderParams;
var
  pr:PGGUIRenderParamsPlane;
begin
  pr:=PGGUIRenderParamsPlane(FRenderParams);
  pr^.GLControlInfo.Lock.Enter;
  try
    pr^.GLControlInfo.GLControl.MakeCurrent();
    FreeAndNil(pr^.Ftexture2D);
    pr^.GLControlInfo.GLControl.ReleaseContext();
  finally
    pr^.GLControlInfo.Lock.Leave;
  end;
  inherited destroyRenderParams;
end;

destructor TGGUIPlane.Destroy;
begin
  inherited Destroy;
end;

procedure TGGUIPlane.Paint;
begin
  paintGUI();
end;

end.

