unit graphica_2d3d;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, graphica_vector, graphica_matrix;

//type


    procedure matrix_transformation(out r:TGMatrix4x4f;constref x:TGVector3f);
    function matrix_transformation(x:TGVector3f):TGMatrix4x4f;inline;
    function matrix_transformation(x,y,z:Single):TGMatrix4x4f;inline;

    procedure vector_transformation(out r:TGVector3f;constref x:TGMatrix4x4f);
    function vector_transformation(x:TGMatrix4x4f):TGVector3f;inline;
    procedure vector_transformation(out r:TGVector4f;constref x:TGMatrix4x4f);
    function vector_transformation(x:TGMatrix4x4f):TGVector4f;inline;

    procedure matrix_scale(out r:TGMatrix4x4f;constref x:TGVector3f);
    function matrix_scale(x:TGVector3f):TGMatrix4x4f;inline;
    function matrix_scale(sx,sy,sz:Single):TGMatrix4x4f;inline;

    procedure vector_scale(out r:TGVector3f;constref x:TGMatrix4x4f);
    function vector_scale(x:TGMatrix4x4f):TGVector3f;inline;



    procedure create_matrix_view(out r:TGMatrix4x4f;constref posX, posY, posZ,
          targetX, targetY, targetZ, upX, upY, upZ:single);
    procedure create_matrix_projection_ortho(out r:TGMatrix4x4f;constref left,
          right, top, bottom, znear,zfar:single);


    function unProject(out r:TGVector4f;constref window:TGVector3f;
                       constref projection,view:TGMatrix4x4f;
                       constref viewport:TGVector4i):Boolean;

    function getWorldToScreen(constref MVP:TGMatrix4x4f;
                              constref viewport:TGVector4i;
                              constref WorldCoordinate:TGVector3f):TGVector2f;
    function getScreenToWorld(constref MVPInvert:PGMatrix4x4f;
                               constref viewport:PGVector4i;
                               constref ScreenCoordinate:PGVector2f;
                               constref zDepth:single):PGVector3f;


    function intersectRayToPlane(out r:TGVector3f; constref AbsoluteMatrix:TGMatrix4x4f;
     constref rayPos,rayN:TGVector4f):boolean;
    function intersectRayToPlane(out r:TGVector3f;
                       constref planePos,planeN,rayPos,rayN:TGVector3f):boolean;


implementation


procedure matrix_transformation(out r: TGMatrix4x4f; constref x: TGVector3f);
begin
  matrix_copy(r,matrix4x4f_one);
  r[0,3]:=x[0];
  r[1,3]:=x[1];
  r[2,3]:=x[2];
end;

function matrix_transformation(x: TGVector3f): TGMatrix4x4f;
begin
  matrix_transformation(Result,x);
end;

function matrix_transformation(x, y, z: Single): TGMatrix4x4f;
begin
  matrix_transformation(Result,vector_set(x,y,z));
end;

procedure vector_transformation(out r: TGVector3f; constref x: TGMatrix4x4f);
begin
  r[0]:=x[0,3];
  r[1]:=x[1,3];
  r[2]:=x[2,3];
end;

function vector_transformation(x: TGMatrix4x4f): TGVector3f;
begin
  vector_transformation(Result,x);
end;

procedure vector_transformation(out r: TGVector4f; constref x: TGMatrix4x4f);
begin
  r[0]:=x[0,3];
  r[1]:=x[1,3];
  r[2]:=x[2,3];
  r[3]:=0;
end;

function vector_transformation(x: TGMatrix4x4f): TGVector4f;
begin
  vector_transformation(Result,x);
end;


procedure matrix_scale(out r: TGMatrix4x4f; constref x: TGVector3f);
begin
  matrix_copy(r,matrix4x4f_one);
  r[0,0]:=x[0];
  r[1,1]:=x[1];
  r[2,2]:=x[2];
end;

function matrix_scale(x: TGVector3f): TGMatrix4x4f;
begin
  matrix_scale(Result,x);
end;

function matrix_scale(sx, sy, sz: Single): TGMatrix4x4f;
begin
  matrix_scale(Result,vector_set(sx,sy,sz));
end;

procedure vector_scale(out r: TGVector3f; constref x: TGMatrix4x4f);
begin
  r[0]:=x[0,0];
  r[1]:=x[1,1];
  r[2]:=x[2,2];
end;

function vector_scale(x: TGMatrix4x4f): TGVector3f;
begin
  vector_scale(Result,x);
end;

procedure create_matrix_view(out r: TGMatrix4x4f; constref posX, posY, posZ,
  targetX, targetY, targetZ, upX, upY, upZ: single);
var
   eye,targer,up,xaxis,yaxis,zaxis:TGVector3f;
begin
 eye:=vector_set(posX,posY,posZ);
 targer:=vector_set(targetX,targetY,targetZ);
 up:=vector_set(upX,upY,upZ);
 up:=vector_normalize(up);
 zaxis:=vector_normalize(eye-targer);
 xaxis:=up><zaxis;
 yaxis:=zaxis><xaxis;
 matrix_copy(r,matrix4x4f_one);

 r[0,0]:=xaxis[0];
 r[0,1]:=xaxis[1];
 r[0,2]:=xaxis[2];
 r[1,0]:=yaxis[0];
 r[1,1]:=yaxis[1];
 r[1,2]:=yaxis[2];
 r[2,0]:=zaxis[0];
 r[2,1]:=zaxis[1];
 r[2,2]:=zaxis[2];
 r[0,3]:=-vector_dot(xaxis,eye);
 r[1,3]:=-vector_dot(yaxis,eye);
 r[2,3]:=-vector_dot(zaxis,eye);

end;

procedure create_matrix_projection_ortho(out r: TGMatrix4x4f; constref left,
  right, top, bottom, znear, zfar: single);
begin
  matrix_copy(r,matrix4x4f_one);
  r[0,0]:=2/(right-left);
  r[1,1]:=2/(top-bottom);
  r[2,2]:=-2/(zfar-znear);
  r[3,0]:=-(right+left)/(right-left);
  r[3,1]:=-(top+bottom)/(top-bottom);
  r[2,3]:=-(zfar+znear)/(zfar-znear);

end;

function unProject(out r: TGVector4f; constref window: TGVector3f; constref
  projection, view: TGMatrix4x4f; constref viewport: TGVector4i): Boolean;
var
  v4f:TGVector4f;
  m4x4f,m4x4Invers:TGMatrix4x4f;
begin
  Result:=false;
  v4f[0]:=2*(window[0]-viewport[0])/viewport[2]-1;
  v4f[1]:=2*(window[1]-viewport[1])/viewport[3]-1;
  v4f[2]:=2*window[2]-1;
  v4f[3]:=1;
  matrix_mul(m4x4f,projection,view);
  matrix_inverse(m4x4Invers,m4x4f);
  matrix_mul(r,m4x4Invers,v4f);
  if(r[3]<>0) then begin
    r:=r/r[3];
    Result:=true;
  end;
end;

function getWorldToScreen(constref MVP: TGMatrix4x4f; constref
  viewport: TGVector4i; constref WorldCoordinate: TGVector3f): TGVector2f;
var
   v4f,screen4f:TGVector4f;
begin
  v4f:=vector_set(WorldCoordinate[0],WorldCoordinate[1],WorldCoordinate[2],0);
  matrix_mul(screen4f,MVP,v4f);
  Result[0]:=(screen4f[0]+1)/2*viewport[2];
  Result[1]:=(1-screen4f[1])/2*viewport[3];
end;

function getScreenToWorld(constref MVPInvert: PGMatrix4x4f; constref
  viewport: PGVector4i; constref ScreenCoordinate: PGVector2f; constref
  zDepth: single): PGVector3f;
begin

end;

function intersectRayToPlane(out r: TGVector3f; constref
  AbsoluteMatrix: TGMatrix4x4f; constref rayPos, rayN: TGVector4f): boolean;
var
   planePosition4f,planeNormal4f:TGVector4f;
   planePosition3f,planeNormal3f,rayPos3f,rayN3f,distance3f:TGVector3f;
   d,a,t:Single;
begin
  Result:=false;
  vector_transformation(planePosition4f,AbsoluteMatrix);
  matrix_mul(planeNormal4f,AbsoluteMatrix,vector4f_normal_plane_xy);
  planeNormal4f:=vector_normalize(planeNormal4f);
  planePosition3f:=planePosition4f;
  planeNormal3f:=planeNormal4f;
  rayPos3f:=rayPos;
  rayN3f:=rayN;

  d:=vector_dot(rayN3f,planeNormal3f);
  distance3f:=planePosition3f-rayPos3f;
  a:=vector_dot(distance3f,planeNormal3f);
  t:=a/d;
  if(t>1e-6) then begin
    r:=rayN3f*t;
    r:=rayPos3f+r;
    r:=r-planePosition3f;
    Result:=true;
  end;
end;

function intersectRayToPlane(out r: TGVector3f; constref planePos, planeN,
  rayPos, rayN: TGVector3f): boolean;
var
   distance3f:TGVector3f;
   d,a,t:Single;
begin
  d:=vector_dot(rayN,planeN);
  distance3f:=planePos-rayPos;
  a:=vector_dot(distance3f,planeN);
  t:=a/d;
  if(t>1e-6) then begin
    r:=rayN*t;
    r:=rayPos+r;
    r:=r-planePos;
    Result:=true;
  end;
end;


end.

