unit graphica_matrix;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, graphica_vector;

type

  TGMatrix2x2f = array [0..1,0..1] of Single;
  PGMatrix2x2f = ^TGMatrix2x2f;

  TGMatrix2x2d = array [0..1,0..1] of Double;
  PGMatrix2x2d = ^TGMatrix2x2d;

  TGMatrix3x3f = array [0..2,0..2] of Single;
  PGMatrix3x3f = ^TGMatrix3x3f;

  TGMatrix3x3d = array [0..2,0..2] of Double;
  PGMatrix3x3d = ^TGMatrix3x3d;

  TGMatrix4x4f = array [0..3,0..3] of Single;
  PGMatrix4x4f = ^TGMatrix4x4f;

  TGMatrix4x4d = array [0..3,0..3] of Double;
  PGMatrix4x4d = ^TGMatrix4x4d;



const
  matrix2x2f_zero: TGMatrix2x2f = ((0,0),(0,0));
  matrix2x2f_one: TGMatrix2x2f = ((1,0),(0,1));
  matrix2x2d_one: TGMatrix2x2d = ((1,0),(0,1));

  matrix3x3f_zero: TGMatrix3x3f = ((0,0,0),(0,0,0),(0,0,0));
  matrix3x3f_one: TGMatrix3x3f = ((1,0,0),(0,1,0),(0,0,1));
  matrix3x3d_one: TGMatrix3x3d = ((1,0,0),(0,1,0),(0,0,1));

  matrix4x4f_zero: TGMatrix4x4f = ((0,0,0,0),(0,0,0,0),(0,0,0,0),(0,0,0,0));
  matrix4x4f_one: TGMatrix4x4f = ((1,0,0,0),(0,1,0,0),(0,0,1,0),(0,0,0,1));
  matrix4x4d_one: TGMatrix4x4d = ((1,0,0,0),(0,1,0,0),(0,0,1,0),(0,0,0,1));


  procedure matrix_add(out r:TGMatrix2x2f; constref x,y:TGMatrix2x2f);
  procedure matrix_add(out r:TGMatrix3x3f; constref x,y:TGMatrix3x3f);
  procedure matrix_add(out r:TGMatrix4x4f; constref x,y:TGMatrix4x4f);
  function matrix_add(x,y:TGMatrix2x2f):TGMatrix2x2f;inline;
  function matrix_add(x,y:TGMatrix3x3f):TGMatrix3x3f;inline;
  function matrix_add(x,y:TGMatrix4x4f):TGMatrix4x4f;inline;
  operator + (x,y:TGMatrix2x2f):TGMatrix2x2f;inline;
  operator + (x,y:TGMatrix3x3f):TGMatrix3x3f;inline;
  operator + (x,y:TGMatrix4x4f):TGMatrix4x4f;inline;

  procedure matrix_sub(out r:TGMatrix2x2f; constref x,y:TGMatrix2x2f);
  procedure matrix_sub(out r:TGMatrix3x3f; constref x,y:TGMatrix3x3f);
  procedure matrix_sub(out r:TGMatrix4x4f; constref x,y:TGMatrix4x4f);
  function matrix_sub(x,y:TGMatrix2x2f):TGMatrix2x2f;inline;
  function matrix_sub(x,y:TGMatrix3x3f):TGMatrix3x3f;inline;
  function matrix_sub(x,y:TGMatrix4x4f):TGMatrix4x4f;inline;
  operator - (x,y:TGMatrix2x2f):TGMatrix2x2f;inline;
  operator - (x,y:TGMatrix3x3f):TGMatrix3x3f;inline;
  operator - (x,y:TGMatrix4x4f):TGMatrix4x4f;inline;


  procedure matrix_mul(out r:TGMatrix2x2f; constref x,y:TGMatrix2x2f);
  procedure matrix_mul(out r:TGMatrix3x3f; constref x,y:TGMatrix3x3f);
  procedure matrix_mul(out r:TGMatrix4x4f; constref x,y:TGMatrix4x4f);
  function matrix_mul(x,y:TGMatrix2x2f):TGMatrix2x2f;inline;
  function matrix_mul(x,y:TGMatrix3x3f):TGMatrix3x3f;inline;
  function matrix_mul(x,y:TGMatrix4x4f):TGMatrix4x4f;inline;
  operator * (x,y:TGMatrix2x2f):TGMatrix2x2f;inline;
  operator * (x,y:TGMatrix3x3f):TGMatrix3x3f;inline;
  operator * (x,y:TGMatrix4x4f):TGMatrix4x4f;inline;

  procedure matrix_add(out r:TGMatrix2x2f; constref x:TGMatrix2x2f;constref y:Single);
  procedure matrix_add(out r:TGMatrix3x3f; constref x:TGMatrix3x3f;constref y:Single);
  procedure matrix_add(out r:TGMatrix4x4f; constref x:TGMatrix4x4f;constref y:Single);
  function matrix_add(x:TGMatrix2x2f;y:Single):TGMatrix2x2f;inline;
  function matrix_add(x:TGMatrix3x3f;y:Single):TGMatrix3x3f;inline;
  function matrix_add(x:TGMatrix4x4f;y:Single):TGMatrix4x4f;inline;
  operator + (x:TGMatrix2x2f;y:Single):TGMatrix2x2f;inline;
  operator + (x:TGMatrix3x3f;y:Single):TGMatrix3x3f;inline;
  operator + (x:TGMatrix4x4f;y:Single):TGMatrix4x4f;inline;

  procedure matrix_sub(out r:TGMatrix2x2f; constref x:TGMatrix2x2f;constref y:Single);
  procedure matrix_sub(out r:TGMatrix3x3f; constref x:TGMatrix3x3f;constref y:Single);
  procedure matrix_sub(out r:TGMatrix4x4f; constref x:TGMatrix4x4f;constref y:Single);
  function matrix_sub(x:TGMatrix2x2f;y:Single):TGMatrix2x2f;inline;
  function matrix_sub(x:TGMatrix3x3f;y:Single):TGMatrix3x3f;inline;
  function matrix_sub(x:TGMatrix4x4f;y:Single):TGMatrix4x4f;inline;
  operator - (x:TGMatrix2x2f;y:Single):TGMatrix2x2f;inline;
  operator - (x:TGMatrix3x3f;y:Single):TGMatrix3x3f;inline;
  operator - (x:TGMatrix4x4f;y:Single):TGMatrix4x4f;inline;

  procedure matrix_mul(out r:TGMatrix2x2f; constref x:TGMatrix2x2f;constref y:Single);
  procedure matrix_mul(out r:TGMatrix3x3f; constref x:TGMatrix3x3f;constref y:Single);
  procedure matrix_mul(out r:TGMatrix4x4f; constref x:TGMatrix4x4f;constref y:Single);
  function matrix_mul(x:TGMatrix2x2f;y:Single):TGMatrix2x2f;inline;
  function matrix_mul(x:TGMatrix3x3f;y:Single):TGMatrix3x3f;inline;
  function matrix_mul(x:TGMatrix4x4f;y:Single):TGMatrix4x4f;inline;
  operator * (x:TGMatrix2x2f;y:Single):TGMatrix2x2f;inline;
  operator * (x:TGMatrix3x3f;y:Single):TGMatrix3x3f;inline;
  operator * (x:TGMatrix4x4f;y:Single):TGMatrix4x4f;inline;

  procedure matrix_div(out r:TGMatrix2x2f; constref x:TGMatrix2x2f;constref y:Single);
  procedure matrix_div(out r:TGMatrix3x3f; constref x:TGMatrix3x3f;constref y:Single);
  procedure matrix_div(out r:TGMatrix4x4f; constref x:TGMatrix4x4f;constref y:Single);
  function matrix_div(x:TGMatrix2x2f;y:Single):TGMatrix2x2f;inline;
  function matrix_div(x:TGMatrix3x3f;y:Single):TGMatrix3x3f;inline;
  function matrix_div(x:TGMatrix4x4f;y:Single):TGMatrix4x4f;inline;
  operator / (x:TGMatrix2x2f;y:Single):TGMatrix2x2f;inline;
  operator / (x:TGMatrix3x3f;y:Single):TGMatrix3x3f;inline;
  operator / (x:TGMatrix4x4f;y:Single):TGMatrix4x4f;inline;

  procedure matrix_copy(out r:TGMatrix2x2f;constref x:TGMatrix2x2f);inline;
  procedure matrix_copy(out r:TGMatrix3x3f;constref x:TGMatrix3x3f);inline;
  procedure matrix_copy(out r:TGMatrix4x4f;constref x:TGMatrix4x4f);inline;
  function matrix_copy(x:TGMatrix2x2f):TGMatrix2x2f;inline;
  function matrix_copy(x:TGMatrix3x3f):TGMatrix3x3f;inline;
  function matrix_copy(x:TGMatrix4x4f):TGMatrix4x4f;inline;


  procedure matrix_mul(out r:TGVector2f; constref x:TGMatrix2x2f;constref y:TGVector2f);
  procedure matrix_mul(out r:TGVector3f; constref x:TGMatrix3x3f;constref y:TGVector3f);
  procedure matrix_mul(out r:TGVector4f; constref x:TGMatrix4x4f;constref y:TGVector4f);
  function matrix_mul(x:TGMatrix2x2f;y:TGVector2f):TGVector2f;inline;
  function matrix_mul(x:TGMatrix3x3f;y:TGVector3f):TGVector3f;inline;
  function matrix_mul(x:TGMatrix4x4f;y:TGVector4f):TGVector4f;inline;
  operator * (x:TGMatrix2x2f;y:TGVector2f):TGVector2f;inline;
  operator * (x:TGMatrix3x3f;y:TGVector3f):TGVector3f;inline;
  operator * (x:TGMatrix4x4f;y:TGVector4f):TGVector4f;inline;


  procedure matrix_determinant(out r:Single;constref x:TGMatrix4x4f);
  function matrix_determinant(x:TGMatrix4x4f):Single;inline;

  procedure matrix_inverse(out r:TGMatrix4x4f;constref x:TGMatrix4x4f);
  function matrix_inverse(x:TGMatrix4x4f):TGMatrix4x4f;inline;

  procedure matrix_transopse(out r:TGMatrix4x4f;constref x:TGMatrix4x4f);
  function matrix_transopse(x:TGMatrix4x4f):TGMatrix4x4f;inline;


implementation

procedure matrix_add(out r:TGMatrix3x3f; constref x,y:TGMatrix3x3f);
begin
  r[0,0]:=x[0,0]+y[0,0];
  r[1,0]:=x[1,0]+y[1,0];
  r[1,1]:=x[1,1]+y[1,1];
  r[0,1]:=x[0,1]+y[0,1];
end;

procedure matrix_add(out r:TGMatrix2x2f; constref x,y:TGMatrix2x2f);
begin
  r[0,0]:=x[0,0]+y[0,0];
  r[0,1]:=x[0,1]+y[0,1];
  r[0,2]:=x[0,2]+y[0,2];
  r[1,0]:=x[1,0]+y[1,0];
  r[1,1]:=x[1,1]+y[1,1];
  r[1,2]:=x[1,2]+y[1,2];
  r[2,0]:=x[2,0]+y[2,0];
  r[2,1]:=x[2,1]+y[2,1];
  r[2,2]:=x[2,2]+y[2,2];
end;

procedure matrix_add(out r:TGMatrix4x4f; constref x,y:TGMatrix4x4f);
begin
  r[0,0]:=x[0,0]+y[0,0];
  r[0,1]:=x[0,1]+y[0,1];
  r[0,2]:=x[0,2]+y[0,2];
  r[0,3]:=x[0,3]+y[0,3];
  r[1,0]:=x[1,0]+y[1,0];
  r[1,1]:=x[1,1]+y[1,1];
  r[1,2]:=x[1,2]+y[1,2];
  r[1,3]:=x[1,3]+y[1,3];
  r[2,0]:=x[2,0]+y[2,0];
  r[2,1]:=x[2,1]+y[2,1];
  r[2,2]:=x[2,2]+y[2,2];
  r[2,3]:=x[2,3]+y[2,3];
  r[3,0]:=x[3,0]+y[3,0];
  r[3,1]:=x[3,1]+y[3,1];
  r[3,2]:=x[3,2]+y[3,2];
  r[3,3]:=x[3,3]+y[3,3];
end;

function matrix_add( x, y: TGMatrix2x2f): TGMatrix2x2f;
begin
  matrix_add(Result,x,y);
end;

function matrix_add( x, y: TGMatrix3x3f): TGMatrix3x3f;
begin
  matrix_add(Result,x,y);
end;

function matrix_add( x, y: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_add(Result,x,y);
end;

operator+(x, y: TGMatrix2x2f): TGMatrix2x2f;
begin
  matrix_add(Result,x,y);
end;

operator+(x, y: TGMatrix3x3f): TGMatrix3x3f;
begin
  matrix_add(Result,x,y);
end;

operator+(x, y: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_add(Result,x,y);
end;

procedure matrix_sub(out r: TGMatrix2x2f; constref x, y: TGMatrix2x2f);
begin
  r[0,0]:=x[0,0]-y[0,0];
  r[0,1]:=x[0,1]-y[0,1];
  r[1,0]:=x[1,0]-y[1,0];
  r[1,1]:=x[1,1]-y[1,1];
end;

procedure matrix_sub(out r: TGMatrix3x3f; constref x, y: TGMatrix3x3f);
begin
  r[0,0]:=x[0,0]-y[0,0];
  r[0,1]:=x[0,1]-y[0,1];
  r[0,2]:=x[0,2]-y[0,2];
  r[1,0]:=x[1,0]-y[1,0];
  r[1,1]:=x[1,1]-y[1,1];
  r[1,2]:=x[1,2]-y[1,2];
  r[2,0]:=x[2,0]-y[2,0];
  r[2,1]:=x[2,1]-y[2,1];
  r[2,2]:=x[2,2]-y[2,2];
end;

procedure matrix_sub(out r: TGMatrix4x4f; constref x, y: TGMatrix4x4f);
begin
  r[0,0]:=x[0,0]-y[0,0];
  r[0,1]:=x[0,1]-y[0,1];
  r[0,2]:=x[0,2]-y[0,2];
  r[0,3]:=x[0,3]-y[0,3];
  r[1,0]:=x[1,0]-y[1,0];
  r[1,1]:=x[1,1]-y[1,1];
  r[1,2]:=x[1,2]-y[1,2];
  r[1,3]:=x[1,3]-y[1,3];
  r[2,0]:=x[2,0]-y[2,0];
  r[2,1]:=x[2,1]-y[2,1];
  r[2,2]:=x[2,2]-y[2,2];
  r[2,3]:=x[2,3]-y[2,3];
  r[3,0]:=x[3,0]-y[3,0];
  r[3,1]:=x[3,1]-y[3,1];
  r[3,2]:=x[3,2]-y[3,2];
  r[3,3]:=x[3,3]-y[3,3];
end;

function matrix_sub(x, y: TGMatrix2x2f): TGMatrix2x2f;
begin
  matrix_sub(Result,x,y);
end;

function matrix_sub(x, y: TGMatrix3x3f): TGMatrix3x3f;
begin
  matrix_sub(Result,x,y);
end;

function matrix_sub(x, y: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_sub(Result,x,y);
end;

operator-(x, y: TGMatrix2x2f): TGMatrix2x2f;
begin
  matrix_sub(Result,x,y);
end;

operator-(x, y: TGMatrix3x3f): TGMatrix3x3f;
begin
  matrix_sub(Result,x,y);
end;

operator-(x, y: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_sub(Result,x,y);
end;


type
  tmatrix_mul_array___=array[0..1] of single;

procedure matrix_mul(out r: TGMatrix2x2f; constref x, y: TGMatrix2x2f);
var v:^tmatrix_mul_array___;
    i,i2:byte;
begin
  New(v);
  try
    for i:=0 to 1 do begin
        for i2:=0 to 1 do
            v^[i2]:=x[i,i2];
        r[i,0]:=v^[0]*y[0,0]
                         +v^[1]*y[1,0];
        r[i,1]:=v^[0]*y[0,1]
                         +v^[1]*y[1,1];
    end;
  finally
    Dispose(v);
  end;
end;

procedure matrix_mul(out r: TGMatrix3x3f; constref x, y: TGMatrix3x3f);
var v:^tmatrix_mul_array___;
    i,i2:byte;
begin
  New(v);
  try
     for i:=0 to 2 do begin
          for i2:=0 to 2 do
              v^[i2]:=x[i,i2];
          r[i,0]:=v^[0]*y[0,0]
                           +v^[1]*y[1,0]
                           +v^[2]*y[2,0];
          r[i,1]:=v^[0]*y[0,1]
                           +v^[1]*y[1,1]
                           +v^[2]*y[2,1];
          r[i,2]:=v^[0]*y[0,2]
                           +v^[1]*y[1,2]
                           +v^[2]*y[2,2];
     end;
  finally
    Dispose(v);
  end;
end;

procedure matrix_mul(out r: TGMatrix4x4f; constref x, y: TGMatrix4x4f);
var v:^tmatrix_mul_array___;
    i,i2:byte;
begin
  New(v);
  try
     for i:=0 to 3 do begin
            for i2:=0 to 3 do
                v^[i2]:=x[i,i2];
            r[i,0]:=v^[0]*y[0,0]
                             +v^[1]*y[1,0]
                             +v^[2]*y[2,0]
                             +v^[3]*y[3,0];
            r[i,1]:=v^[0]*y[0,1]
                             +v^[1]*y[1,1]
                             +v^[2]*y[2,1]
                             +v^[3]*y[3,1];
            r[i,2]:=v^[0]*y[0,2]
                             +v^[1]*y[1,2]
                             +v^[2]*y[2,2]
                             +v^[3]*y[3,2];
            r[i,3]:=v^[0]*y[0,3]
                             +v^[1]*y[1,3]
                             +v^[2]*y[2,3]
                             +v^[3]*y[3,3];
     end;
  finally
    Dispose(v);
  end;
end;

function matrix_mul(x, y: TGMatrix2x2f): TGMatrix2x2f;
begin
  matrix_mul(Result,x,y);
end;

function matrix_mul(x, y: TGMatrix3x3f): TGMatrix3x3f;
begin
  matrix_mul(Result,x,y);
end;

function matrix_mul(x, y: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x, y: TGMatrix2x2f): TGMatrix2x2f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x, y: TGMatrix3x3f): TGMatrix3x3f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x, y: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_mul(Result,x,y);
end;

procedure matrix_add(out r: TGMatrix2x2f; constref x: TGMatrix2x2f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]+y;
  r[0,1]:=x[0,1]+y;
  r[1,0]:=x[1,0]+y;
  r[1,1]:=x[1,1]+y;
end;

procedure matrix_add(out r: TGMatrix3x3f; constref x: TGMatrix3x3f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]+y;
  r[0,1]:=x[0,1]+y;
  r[0,2]:=x[0,2]+y;
  r[1,0]:=x[1,0]+y;
  r[1,1]:=x[1,1]+y;
  r[1,2]:=x[1,2]+y;
  r[2,0]:=x[2,0]+y;
  r[2,1]:=x[2,1]+y;
  r[2,2]:=x[2,2]+y;
end;

procedure matrix_add(out r: TGMatrix4x4f; constref x: TGMatrix4x4f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]+y;
  r[0,1]:=x[0,1]+y;
  r[0,2]:=x[0,2]+y;
  r[0,3]:=x[0,3]+y;
  r[1,0]:=x[1,0]+y;
  r[1,1]:=x[1,1]+y;
  r[1,2]:=x[1,2]+y;
  r[1,3]:=x[1,3]+y;
  r[2,0]:=x[2,0]+y;
  r[2,1]:=x[2,1]+y;
  r[2,2]:=x[2,2]+y;
  r[2,3]:=x[2,3]+y;
  r[3,0]:=x[3,0]+y;
  r[3,1]:=x[3,1]+y;
  r[3,2]:=x[3,2]+y;
  r[3,3]:=x[3,3]+y;
end;

function matrix_add(x: TGMatrix2x2f; y: Single): TGMatrix2x2f;
begin
  matrix_add(Result,x,y);
end;

function matrix_add(x: TGMatrix3x3f; y: Single): TGMatrix3x3f;
begin
  matrix_add(Result,x,y);
end;

function matrix_add(x: TGMatrix4x4f; y: Single): TGMatrix4x4f;
begin
  matrix_add(Result,x,y);
end;

operator+(x: TGMatrix2x2f; y: Single): TGMatrix2x2f;
begin
  matrix_add(Result,x,y);
end;

operator+(x: TGMatrix3x3f; y: Single): TGMatrix3x3f;
begin
  matrix_add(Result,x,y);
end;

operator+(x: TGMatrix4x4f; y: Single): TGMatrix4x4f;
begin
  matrix_add(Result,x,y);
end;

procedure matrix_sub(out r: TGMatrix2x2f; constref x: TGMatrix2x2f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]-y;
  r[0,1]:=x[0,1]-y;
  r[1,0]:=x[1,0]-y;
  r[1,1]:=x[1,1]-y;
end;

procedure matrix_sub(out r: TGMatrix3x3f; constref x: TGMatrix3x3f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]-y;
  r[0,1]:=x[0,1]-y;
  r[0,2]:=x[0,2]-y;
  r[1,0]:=x[1,0]-y;
  r[1,1]:=x[1,1]-y;
  r[1,2]:=x[1,2]-y;
  r[2,0]:=x[2,0]-y;
  r[2,1]:=x[2,1]-y;
  r[2,2]:=x[2,2]-y;
end;

procedure matrix_sub(out r: TGMatrix4x4f; constref x: TGMatrix4x4f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]-y;
  r[0,1]:=x[0,1]-y;
  r[0,2]:=x[0,2]-y;
  r[0,3]:=x[0,3]-y;
  r[1,0]:=x[1,0]-y;
  r[1,1]:=x[1,1]-y;
  r[1,2]:=x[1,2]-y;
  r[1,3]:=x[1,3]-y;
  r[2,0]:=x[2,0]-y;
  r[2,1]:=x[2,1]-y;
  r[2,2]:=x[2,2]-y;
  r[2,3]:=x[2,3]-y;
  r[3,0]:=x[3,0]-y;
  r[3,1]:=x[3,1]-y;
  r[3,2]:=x[3,2]-y;
  r[3,3]:=x[3,3]-y;
end;

function matrix_sub(x: TGMatrix2x2f; y: Single): TGMatrix2x2f;
begin
  matrix_sub(Result,x,y);
end;

function matrix_sub(x: TGMatrix3x3f; y: Single): TGMatrix3x3f;
begin
  matrix_sub(Result,x,y);
end;

function matrix_sub(x: TGMatrix4x4f; y: Single): TGMatrix4x4f;
begin
  matrix_sub(Result,x,y);
end;

operator-(x: TGMatrix2x2f; y: Single): TGMatrix2x2f;
begin
  matrix_sub(Result,x,y);
end;

operator-(x: TGMatrix3x3f; y: Single): TGMatrix3x3f;
begin
  matrix_sub(Result,x,y);
end;

operator-(x: TGMatrix4x4f; y: Single): TGMatrix4x4f;
begin
  matrix_sub(Result,x,y);
end;

procedure matrix_mul(out r: TGMatrix2x2f; constref x: TGMatrix2x2f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]*y;
  r[0,1]:=x[0,1]*y;
  r[1,0]:=x[1,0]*y;
  r[1,1]:=x[1,1]*y;
end;

procedure matrix_mul(out r: TGMatrix3x3f; constref x: TGMatrix3x3f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]*y;
  r[0,1]:=x[0,1]*y;
  r[0,2]:=x[0,2]*y;
  r[1,0]:=x[1,0]*y;
  r[1,1]:=x[1,1]*y;
  r[1,2]:=x[1,2]*y;
  r[2,0]:=x[2,0]*y;
  r[2,1]:=x[2,1]*y;
  r[2,2]:=x[2,2]*y;
end;

procedure matrix_mul(out r: TGMatrix4x4f; constref x: TGMatrix4x4f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]*y;
  r[0,1]:=x[0,1]*y;
  r[0,2]:=x[0,2]*y;
  r[0,3]:=x[0,3]*y;
  r[1,0]:=x[1,0]*y;
  r[1,1]:=x[1,1]*y;
  r[1,2]:=x[1,2]*y;
  r[1,3]:=x[1,3]*y;
  r[2,0]:=x[2,0]*y;
  r[2,1]:=x[2,1]*y;
  r[2,2]:=x[2,2]*y;
  r[2,3]:=x[2,3]*y;
  r[3,0]:=x[3,0]*y;
  r[3,1]:=x[3,1]*y;
  r[3,2]:=x[3,2]*y;
  r[3,3]:=x[3,3]*y;
end;

function matrix_mul(x: TGMatrix2x2f; y: Single): TGMatrix2x2f;
begin
  matrix_mul(Result,x,y);
end;

function matrix_mul(x: TGMatrix3x3f; y: Single): TGMatrix3x3f;
begin
  matrix_mul(Result,x,y);
end;

function matrix_mul(x: TGMatrix4x4f; y: Single): TGMatrix4x4f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x: TGMatrix2x2f; y: Single): TGMatrix2x2f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x: TGMatrix3x3f; y: Single): TGMatrix3x3f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x: TGMatrix4x4f; y: Single): TGMatrix4x4f;
begin
  matrix_mul(Result,x,y);
end;

procedure matrix_div(out r: TGMatrix2x2f; constref x: TGMatrix2x2f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]/y;
  r[0,1]:=x[0,1]/y;
  r[1,0]:=x[1,0]/y;
  r[1,1]:=x[1,1]/y;
end;

procedure matrix_div(out r: TGMatrix3x3f; constref x: TGMatrix3x3f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]/y;
  r[0,1]:=x[0,1]/y;
  r[0,2]:=x[0,2]/y;
  r[1,0]:=x[1,0]/y;
  r[1,1]:=x[1,1]/y;
  r[1,2]:=x[1,2]/y;
  r[2,0]:=x[2,0]/y;
  r[2,1]:=x[2,1]/y;
  r[2,2]:=x[2,2]/y;
end;

procedure matrix_div(out r: TGMatrix4x4f; constref x: TGMatrix4x4f; constref
  y: Single);
begin
  r[0,0]:=x[0,0]/y;
  r[0,1]:=x[0,1]/y;
  r[0,2]:=x[0,2]/y;
  r[0,3]:=x[0,3]/y;
  r[1,0]:=x[1,0]/y;
  r[1,1]:=x[1,1]/y;
  r[1,2]:=x[1,2]/y;
  r[1,3]:=x[1,3]/y;
  r[2,0]:=x[2,0]/y;
  r[2,1]:=x[2,1]/y;
  r[2,2]:=x[2,2]/y;
  r[2,3]:=x[2,3]/y;
  r[3,0]:=x[3,0]/y;
  r[3,1]:=x[3,1]/y;
  r[3,2]:=x[3,2]/y;
  r[3,3]:=x[3,3]/y;
end;

function matrix_div(x: TGMatrix2x2f; y: Single): TGMatrix2x2f;
begin
  matrix_div(Result,x,y);
end;

function matrix_div(x: TGMatrix3x3f; y: Single): TGMatrix3x3f;
begin
  matrix_div(Result,x,y);
end;

function matrix_div(x: TGMatrix4x4f; y: Single): TGMatrix4x4f;
begin
  matrix_div(Result,x,y);
end;

operator/(x: TGMatrix2x2f; y: Single): TGMatrix2x2f;
begin
  matrix_div(Result,x,y);
end;

operator/(x: TGMatrix3x3f; y: Single): TGMatrix3x3f;
begin
  matrix_div(Result,x,y);
end;

operator/(x: TGMatrix4x4f; y: Single): TGMatrix4x4f;
begin
  matrix_div(Result,x,y);
end;

procedure matrix_copy(out r: TGMatrix2x2f; constref x: TGMatrix2x2f);
begin
end;

procedure matrix_copy(out r: TGMatrix3x3f; constref x: TGMatrix3x3f);
begin
  Move(x,r,SizeOf(TGMatrix3x3f));
end;

procedure matrix_copy(out r: TGMatrix4x4f; constref x: TGMatrix4x4f);
begin
  Move(x,r,SizeOf(TGMatrix4x4f));
end;

function matrix_copy(x: TGMatrix2x2f): TGMatrix2x2f;
begin
  matrix_copy(result,x);
end;

function matrix_copy(x: TGMatrix3x3f): TGMatrix3x3f;
begin
  matrix_copy(Result,x);
end;

function matrix_copy(x: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_copy(Result,x);
end;

procedure matrix_mul(out r: TGVector2f; constref x: TGMatrix2x2f; constref
  y: TGVector2f);
begin
  r[0]:=x[0,0]*y[0]
                 +x[0,1]*y[1];
  r[1]:=x[1,0]*y[0]
                 +x[1,1]*y[1];
end;

procedure matrix_mul(out r: TGVector3f; constref x: TGMatrix3x3f; constref
  y: TGVector3f);
begin
  r[0]:=x[0,0]*y[0]
                 +x[0,1]*y[1]
                 +x[0,2]*y[2];
  r[1]:=x[1,0]*y[0]
                 +x[1,1]*y[1]
                 +x[1,2]*y[2];
  r[2]:=x[2,0]*y[0]
                 +x[2,1]*y[1]
                 +x[2,2]*y[2];
end;

procedure matrix_mul(out r: TGVector4f; constref x: TGMatrix4x4f; constref
  y: TGVector4f);
begin
  r[0]:=x[0,0]*y[0]+x[0,1]*y[1]+x[0,2]*y[2]+x[0,3]*y[3];
  r[1]:=x[1,0]*y[0]+x[1,1]*y[1]+x[1,2]*y[2]+x[1,3]*y[3];
  r[2]:=x[2,0]*y[0]+x[2,1]*y[1]+x[2,2]*y[2]+x[2,3]*y[3];
  r[3]:=x[3,0]*y[0]+x[3,1]*y[1]+x[3,2]*y[2]+x[3,3]*y[3];
end;

function matrix_mul(x: TGMatrix2x2f; y: TGVector2f): TGVector2f;
begin
  matrix_mul(Result,x,y);
end;

function matrix_mul(x: TGMatrix3x3f; y: TGVector3f): TGVector3f;
begin
  matrix_mul(Result,x,y);
end;

function matrix_mul(x: TGMatrix4x4f; y: TGVector4f): TGVector4f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x: TGMatrix2x2f; y: TGVector2f): TGVector2f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x: TGMatrix3x3f; y: TGVector3f): TGVector3f;
begin
  matrix_mul(Result,x,y);
end;

operator*(x: TGMatrix4x4f; y: TGVector4f): TGVector4f;
begin
  matrix_mul(Result,x,y);
end;

procedure matrix_determinant(out r: Single; constref x: TGMatrix4x4f);
begin
  r:=
      x[0,0]*x[1,1]*x[2,2]*x[3,3]+x[0,0]*x[1,2]*x[2,3]*x[3,1]+
                x[0,0]*x[1,3]*x[2,1]*x[3,2]
      +x[0,1]*x[1,0]*x[2,3]*x[3,2]+x[0,1]*x[1,2]*x[2,0]*x[3,3]+
                x[0,1]*x[1,3]*x[2,2]*x[3,0]
      +x[0,2]*x[1,0]*x[2,1]*x[3,3]+x[0,2]*x[1,1]*x[2,3]*x[3,0]+
                x[0,2]*x[1,3]*x[2,0]*x[3,1]
      +x[0,3]*x[1,0]*x[2,2]*x[3,1]+x[0,3]*x[1,1]*x[2,0]*x[3,2]+
                x[0,3]*x[1,2]*x[2,1]*x[3,0]
      -x[0,0]*x[1,1]*x[2,3]*x[3,2]-x[0,0]*x[1,2]*x[2,1]*x[3,3]-
                 x[0,0]*x[1,3]*x[2,2]*x[3,1]
      -x[0,1]*x[1,0]*x[2,2]*x[3,3]-x[0,1]*x[1,2]*x[2,3]*x[3,0]-
                x[0,1]*x[1,3]*x[2,0]*x[3,2]
      -x[0,2]*x[1,0]*x[2,3]*x[3,1]-x[0,2]*x[1,1]*x[2,0]*x[3,3]-
                 x[0,2]*x[1,3]*x[2,1]*x[3,0]
      -x[0,3]*x[1,0]*x[2,1]*x[3,2]-x[0,3]*x[1,1]*x[2,2]*x[3,0]-
                x[0,3]*x[1,2]*x[2,0]*x[3,1];
end;

function matrix_determinant(x: TGMatrix4x4f): Single;
begin
  matrix_determinant(Result,x);
end;

procedure matrix_inverse(out r: TGMatrix4x4f; constref x: TGMatrix4x4f);
type
  _m=record
    b:TGMatrix4x4f;
    det:Single;
  end;
var
  b:^_m;
begin
  b:=GetMem(SizeOf(_m));
  try
    b^.det:=matrix_determinant(x);
    b^.b[0,0]:=
              x[1,1]*x[2,2]*x[3,3]+x[1,2]*x[2,3]*x[3,1]+
                             x[1,3]*x[2,1]*x[3,2]
             -x[1,1]*x[2,3]*x[3,2]-x[1,2]*x[2,1]*x[3,3]-
                             x[1,3]*x[2,2]*x[3,1];
    b^.b[0,1]:=
              x[0,1]*x[2,3]*x[3,2]+x[0,2]*x[2,1]*x[3,3]+
                             x[0,3]*x[2,2]*x[3,1]
             -x[0,1]*x[2,2]*x[3,3]-x[0,2]*x[2,3]*x[3,1]-
                             x[0,3]*x[2,1]*x[3,2];
    b^.b[0,2]:=
              x[0,1]*x[1,2]*x[3,3]+x[0,2]*x[1,3]*x[3,1]+
                             x[0,3]*x[1,1]*x[3,2]
             -x[0,1]*x[1,3]*x[3,2]-x[0,2]*x[1,1]*x[3,3]-
                             x[0,3]*x[1,2]*x[3,1];
    b^.b[0,3]:=
              x[0,1]*x[1,3]*x[2,2]+x[0,2]*x[1,1]*x[2,3]+
                             x[0,3]*x[1,2]*x[2,1]
             -x[0,1]*x[1,2]*x[2,3]-x[0,2]*x[1,3]*x[2,1]-
                             x[0,3]*x[1,1]*x[2,2];
    b^.b[1,0]:=
              x[1,0]*x[2,3]*x[3,2]+x[1,2]*x[2,0]*x[3,3]+
                             x[1,3]*x[2,2]*x[3,0]
             -x[1,0]*x[2,2]*x[3,3]-x[1,2]*x[2,3]*x[3,0]-
                             x[1,3]*x[2,0]*x[3,2];
    b^.b[1,1]:=
              x[0,0]*x[2,2]*x[3,3]+x[0,2]*x[2,3]*x[3,0]+
                             x[0,3]*x[2,0]*x[3,2]
             -x[0,0]*x[2,3]*x[3,2]-x[0,2]*x[2,0]*x[3,3]-
                             x[0,3]*x[2,2]*x[3,0];
    b^.b[1,2]:=
              x[0,0]*x[1,3]*x[3,2]+x[0,2]*x[1,0]*x[3,3]+
                             x[0,3]*x[1,2]*x[3,0]
             -x[0,0]*x[1,2]*x[3,3]-x[0,2]*x[1,3]*x[3,0]-
                             x[0,3]*x[1,0]*x[3,2];
    b^.b[1,3]:=
              x[0,0]*x[1,2]*x[2,3]+x[0,2]*x[1,3]*x[2,0]+
                             x[0,3]*x[1,0]*x[2,2]
             -x[0,0]*x[1,3]*x[2,2]-x[0,2]*x[1,0]*x[2,3]-
                             x[0,3]*x[1,2]*x[2,0];

     b^.b[2,0]:=
              x[1,0]*x[2,1]*x[3,3]+x[1,1]*x[2,3]*x[3,0]+
                             x[1,3]*x[2,0]*x[3,1]
             -x[1,0]*x[2,3]*x[3,1]-x[1,1]*x[2,0]*x[3,3]-
                             x[1,3]*x[2,1]*x[3,0];
     b^.b[2,1]:=
              x[0,0]*x[2,3]*x[3,1]+x[0,1]*x[2,0]*x[3,3]+
                             x[0,3]*x[2,1]*x[3,0]
             -x[0,0]*x[2,1]*x[3,3]-x[0,1]*x[2,3]*x[3,0]-
                             x[0,3]*x[2,0]*x[3,1];
     b^.b[2,2]:=
              x[0,0]*x[1,1]*x[3,3]+x[0,1]*x[1,3]*x[3,0]+
                             x[0,3]*x[1,0]*x[3,1]
             -x[0,0]*x[1,3]*x[3,1]-x[0,1]*x[1,0]*x[3,3]-
                             x[0,3]*x[1,1]*x[3,0];
     b^.b[2,3]:=
              x[0,0]*x[1,3]*x[2,1]+x[0,1]*x[1,0]*x[2,3]+
                             x[0,3]*x[1,1]*x[2,0]
             -x[0,0]*x[1,1]*x[2,3]-x[0,1]*x[1,3]*x[2,0]-
                             x[0,3]*x[1,0]*x[2,1];

     b^.b[3,0]:=
              x[1,0]*x[2,2]*x[3,1]+x[1,1]*x[2,0]*x[3,2]+
                             x[1,2]*x[2,1]*x[3,0]
             -x[1,0]*x[2,1]*x[3,2]-x[1,1]*x[2,2]*x[3,0]-
                             x[1,2]*x[2,0]*x[3,1];
     b^.b[3,1]:=
              x[0,0]*x[2,1]*x[3,2]+x[0,1]*x[2,2]*x[3,0]+
                             x[0,2]*x[2,0]*x[3,1]
             -x[0,0]*x[2,2]*x[3,1]-x[0,1]*x[2,0]*x[3,2]-
                             x[0,2]*x[2,1]*x[3,0];
     b^.b[3,2]:=
              x[0,0]*x[1,2]*x[3,1]+x[0,1]*x[1,0]*x[3,2]+
                             x[0,2]*x[1,1]*x[3,0]
             -x[0,0]*x[1,1]*x[3,2]-x[0,1]*x[1,2]*x[3,0]-
                             x[0,2]*x[1,0]*x[3,1];
     b^.b[3,3]:=
              x[0,0]*x[1,1]*x[2,2]+x[0,1]*x[1,2]*x[2,0]+
                             x[0,2]*x[1,0]*x[2,1]
             -x[0,0]*x[1,2]*x[2,1]-x[0,1]*x[1,0]*x[2,2]-
                             x[0,2]*x[1,1]*x[2,0];
     matrix_div(r,b^.b,b^.det);
  finally
    Freemem(b);
  end;
end;

function matrix_inverse(x: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_inverse(Result,x);
end;

procedure matrix_transopse(out r: TGMatrix4x4f; constref x: TGMatrix4x4f);
var
  i,j:Integer;
begin
  for i:=0 to 3 do
      for j:=0 to 3 do
          r[i,j]:=x[j,i];
end;

function matrix_transopse(x: TGMatrix4x4f): TGMatrix4x4f;
begin
  matrix_transopse(Result,x);
end;

end.

