unit graphica_vector;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;
type

  TGVector1i = Integer;
  PGVector1i = ^TGVector1i;
  TGVector1f = Single;
  PGVector1f = ^TGVector1f;
  TGVector1d = Double;
  PGVector1d = ^TGVector1d;

  TGVector2i = array [0..1] of Integer;
  PGVector2i = ^TGVector2i;
  TGVector2f = array [0..1] of Single;
  PGVector2f = ^TGVector2f;
  TGVector2d = array [0..1] of Double;
  PGVector2d = ^TGVector2d;

  TGVector3i = array [0..2] of Integer;
  PGVector3i = ^TGVector3i;
  TGVector3f = array [0..2] of Single;
  PGVector3f = ^TGVector3f;
  TGVector3d = array [0..2] of Double;
  PGVector3d = ^TGVector3d;

  TGVector4b = array [0..3] of byte;
  PGVector4b = ^TGVector4b;
  TGVector4i = array [0..3] of Integer;
  PGVector4i = ^TGVector4i;
  TGVector4f = array [0..3] of Single;
  PGVector4f = ^TGVector4f;
  TGVector4d = array [0..3] of Double;
  PGVector4d = ^TGVector4d;

const
  vector2f_zero:TGVector2f=(0,0);
  vector3f_zero:TGVector3f=(0,0,0);
  vector4f_zero:TGVector4f=(0,0,0,0);
  vector2f_one:TGVector2f=(1,1);
  vector3f_one:TGVector3f=(1,1,1);
  vector4f_one:TGVector4f=(1,1,1,1);
  vector4f_x_coordinate:TGVector4f=(1,0,0,0);
  vector4f_y_coordinate:TGVector4f=(0,1,0,0);
  vector4f_z_coordinate:TGVector4f=(0,0,1,0);
  vector3f_normal_plane_xy:TGVector3f=(0,0,-1);
  vector3f_normal_plane_yz:TGVector3f=(-1,0,0);
  vector3f_normal_plane_xz:TGVector3f=(0,-1,0);
  vector4f_normal_plane_xy:TGVector4f=(0,0,-1,0);


  operator := (x:TGVector4f):TGVector2f;inline;
  operator := (x:TGVector4f):TGVector3f;inline;


  operator + (x,y:TGVector2f):TGVector2f;inline;
  operator + (x,y:TGVector3f):TGVector3f;inline;
  operator + (x,y:TGVector4f):TGVector4f;inline;

  operator - (x,y:TGVector2f):TGVector2f;inline;
  operator - (x,y:TGVector3f):TGVector3f;inline;
  operator - (x,y:TGVector4f):TGVector4f;inline;

  operator * (x,y:TGVector2f):TGVector2f;inline;
  operator * (x,y:TGVector3f):TGVector3f;inline;
  operator * (x,y:TGVector4f):TGVector4f;inline;

  operator / (x,y:TGVector2f):TGVector2f;inline;
  operator / (x,y:TGVector3f):TGVector3f;inline;
  operator / (x,y:TGVector4f):TGVector4f;inline;

  operator + (x:TGVector2f;y:Single):TGVector2f;inline;
  operator + (x:TGVector3f;y:Single):TGVector3f;inline;
  operator + (x:TGVector4f;y:Single):TGVector4f;inline;

  operator - (x:TGVector2f;y:Single):TGVector2f;inline;
  operator - (x:TGVector3f;y:Single):TGVector3f;inline;
  operator - (x:TGVector4f;y:Single):TGVector4f;inline;

  operator * (x:TGVector2f;y:Single):TGVector2f;inline;
  operator * (x:TGVector3f;y:Single):TGVector3f;inline;
  operator * (x:TGVector4f;y:Single):TGVector4f;inline;

  operator / (x:TGVector2f;y:Single):TGVector2f;inline;
  operator / (x:TGVector3f;y:Single):TGVector3f;inline;
  operator / (x:TGVector4f;y:Single):TGVector4f;inline;


  procedure vector_dot(out r:Single;constref x,y:TGVector2f);
  procedure vector_dot(out r:Single;constref x,y:TGVector3f);
  procedure vector_dot(out r:Single;constref x,y:TGVector4f);
  function vector_dot(x,y:TGVector2f):Single;inline;
  function vector_dot(x,y:TGVector3f):Single;inline;
  function vector_dot(x,y:TGVector4f):Single;inline;
  operator **(x,y:TGVector2f):TGVector2f;inline;
  operator **(x,y:TGVector3f):TGVector3f;inline;
  operator **(x,y:TGVector4f):TGVector4f;inline;

  procedure vector_cross(out r:TGVector3f;constref x,y:TGVector3f);
  function vector_cross(x,y:TGVector3f):TGVector3f;inline;
  operator ><(x,y:TGVector3f):TGVector3f;inline;

  procedure vector_length(out r:Single;constref x:TGVector2f);
  procedure vector_length(out r:Single;constref x:TGVector3f);
  procedure vector_length(out r:Single;constref x:TGVector4f);
  function  vector_length(x:TGVector2f):Single;inline;
  function  vector_length(x:TGVector3f):Single;inline;
  function  vector_length(x:TGVector4f):Single;inline;


  procedure vector_normalize(out r:TGVector2f;constref x:TGVector2f);
  procedure vector_normalize(out r:TGVector3f;constref x:TGVector3f);
  procedure vector_normalize(out r:TGVector4f;constref x:TGVector4f);
  function  vector_normalize(x:TGVector2f):TGVector2f;inline;
  function  vector_normalize(x:TGVector3f):TGVector3f;inline;
  function  vector_normalize(x:TGVector4f):TGVector4f;inline;

  procedure vector_copy(out r:TGVector2f;constref x:TGVector2f);
  procedure vector_copy(out r:TGVector3f;constref x:TGVector3f);
  procedure vector_copy(out r:TGVector4f;constref x:TGVector4f);
  procedure vector_copy(out r:TGVector4i;constref x:TGVector4i);

  procedure vector_set(out r:TGVector2f;constref x0,x1:Single);
  procedure vector_set(out r:TGVector3f;constref x0,x1,x2:Single);
  procedure vector_set(out r:TGVector4f;constref x0,x1,x2,x3:Single);
  function  vector_set(x0,x1:Single):TGVector2f;inline;
  function  vector_set(x0,x1,x2:Single):TGVector3f;inline;
  function  vector_set(x0,x1,x2,x3:Single):TGVector4f;inline;

  procedure vector_negative(out r:TGVector2f;constref x:TGVector2f);
  procedure vector_negative(out r:TGVector3f;constref x:TGVector3f);
  procedure vector_negative(out r:TGVector4f;constref x:TGVector4f);
  function  vector_negative(x:TGVector2f):TGVector2f;inline;
  function  vector_negative(x:TGVector3f):TGVector3f;inline;
  function  vector_negative(x:TGVector4f):TGVector4f;inline;



implementation

operator:=(x: TGVector4f): TGVector2f;
begin
  result[0]:=x[0];
  result[1]:=x[1];
end;

operator:=(x: TGVector4f): TGVector3f;
begin
  result[0]:=x[0];
  result[1]:=x[1];
  result[2]:=x[2];
end;

operator+(x, y: TGVector2f): TGVector2f;
begin
  result[0]:=x[0]+y[0];
  result[1]:=x[1]+y[1];
end;

operator+(x, y: TGVector3f): TGVector3f;
begin
  result[0]:=x[0]+y[0];
  result[1]:=x[1]+y[1];
  result[2]:=x[2]+y[2];
end;

operator+(x, y: TGVector4f): TGVector4f;
begin
  result[0]:=x[0]+y[0];
  result[1]:=x[1]+y[1];
  result[2]:=x[2]+y[2];
  result[3]:=x[3]+y[3];
end;

operator-(x, y: TGVector2f): TGVector2f;
begin
  result[0]:=x[0]-y[0];
  result[1]:=x[1]-y[1];
end;

operator-(x, y: TGVector3f): TGVector3f;
begin
  result[0]:=x[0]-y[0];
  result[1]:=x[1]-y[1];
  result[2]:=x[2]-y[2];
end;

operator-(x, y: TGVector4f): TGVector4f;
begin
  result[0]:=x[0]-y[0];
  result[1]:=x[1]-y[1];
  result[2]:=x[2]-y[2];
  result[3]:=x[3]-y[3];
end;

operator*(x, y: TGVector2f): TGVector2f;
begin
  result[0]:=x[0]*y[0];
  result[1]:=x[1]*y[1];
end;

operator*(x, y: TGVector3f): TGVector3f;
begin
  result[0]:=x[0]*y[0];
  result[1]:=x[1]*y[1];
  result[2]:=x[2]*y[2];
end;

operator*(x, y: TGVector4f): TGVector4f;
begin
  result[0]:=x[0]*y[0];
  result[1]:=x[1]*y[1];
  result[2]:=x[2]*y[2];
  result[3]:=x[3]*y[3];
end;

operator/(x, y: TGVector2f): TGVector2f;
begin
  result[0]:=x[0]/y[0];
  result[1]:=x[1]/y[1];
end;

operator/(x, y: TGVector3f): TGVector3f;
begin
  result[0]:=x[0]/y[0];
  result[1]:=x[1]/y[1];
  result[2]:=x[2]/y[2];
end;

operator/(x, y: TGVector4f): TGVector4f;
begin
  result[0]:=x[0]/y[0];
  result[1]:=x[1]/y[1];
  result[2]:=x[2]/y[2];
  result[3]:=x[3]/y[3];
end;

operator+(x: TGVector2f; y: Single): TGVector2f;
begin
  result[0]:=x[0]+y;
  result[1]:=x[1]+y;
end;

operator+(x: TGVector3f; y: Single): TGVector3f;
begin
  result[0]:=x[0]+y;
  result[1]:=x[1]+y;
  result[2]:=x[2]+y;
end;

operator+(x: TGVector4f; y: Single): TGVector4f;
begin
  result[0]:=x[0]+y;
  result[1]:=x[1]+y;
  result[2]:=x[2]+y;
  result[3]:=x[3]+y;
end;

operator-(x: TGVector2f; y: Single): TGVector2f;
begin
  result[0]:=x[0]-y;
  result[1]:=x[1]-y;
end;

operator-(x: TGVector3f; y: Single): TGVector3f;
begin
  result[0]:=x[0]-y;
  result[1]:=x[1]-y;
  result[2]:=x[2]-y;
end;

operator-(x: TGVector4f; y: Single): TGVector4f;
begin
  result[0]:=x[0]-y;
  result[1]:=x[1]-y;
  result[2]:=x[2]-y;
  result[3]:=x[3]-y;
end;

operator*(x: TGVector2f; y: Single): TGVector2f;
begin
  result[0]:=x[0]*y;
  result[1]:=x[1]*y;
end;

operator*(x: TGVector3f; y: Single): TGVector3f;
begin
  result[0]:=x[0]*y;
  result[1]:=x[1]*y;
  result[2]:=x[2]*y;
end;

operator*(x: TGVector4f; y: Single): TGVector4f;
begin
  result[0]:=x[0]*y;
  result[1]:=x[1]*y;
  result[2]:=x[2]*y;
  result[3]:=x[3]*y;
end;

operator/(x: TGVector2f; y: Single): TGVector2f;
begin
  result[0]:=x[0]/y;
  result[1]:=x[1]/y;
end;

operator/(x: TGVector3f; y: Single): TGVector3f;
begin
  result[0]:=x[0]/y;
  result[1]:=x[1]/y;
  result[2]:=x[2]/y;
end;

operator/(x: TGVector4f; y: Single): TGVector4f;
begin
  result[0]:=x[0]/y;
  result[1]:=x[1]/y;
  result[2]:=x[2]/y;
  result[3]:=x[3]/y;
end;


procedure vector_dot(out r: Single; constref x, y: TGVector2f);
begin
  r:=x[0]*y[0]
      +x[1]*y[1];
end;

procedure vector_dot(out r: Single; constref x, y: TGVector3f);
begin
  r:=x[0]*y[0]
      +x[1]*y[1]
      +x[2]*y[2];
end;

procedure vector_dot(out r: Single; constref x, y: TGVector4f);
begin
  r:=x[0]*y[0]
      +x[1]*y[1]
      +x[2]*y[2]
      +x[3]*y[3];
end;

function vector_dot(x, y: TGVector2f): Single;
begin
  vector_dot(Result,x,y);
end;

function vector_dot(x, y: TGVector3f): Single;
begin
  vector_dot(Result,x,y);
end;

function vector_dot(x, y: TGVector4f): Single;
begin
  vector_dot(Result,x,y);
end;

operator**(x, y: TGVector2f): TGVector2f;
begin
//  vector_dot(Result,x,y);
end;

operator**(x, y: TGVector3f): TGVector3f;
begin
//  vector_dot(Result,x,y);
end;

operator**(x, y: TGVector4f): TGVector4f;
begin
//  vector_dot(Result,x,y);
end;

procedure vector_cross(out r: TGVector3f; constref x, y: TGVector3f);
begin
  r[0]:=x[1]*y[2]-x[2]*y[1];
  r[1]:=x[2]*y[0]-x[0]*y[2];
  r[2]:=x[0]*y[1]-x[1]*y[0];
end;

function vector_cross(x, y: TGVector3f): TGVector3f;
begin
  vector_cross(Result,x,y);
end;

operator><(x, y: TGVector3f): TGVector3f;
begin
  vector_cross(Result,x,y);
end;

procedure vector_length(out r: Single; constref x: TGVector2f);
begin
  r:=sqrt(x[0]*x[0]
         +x[1]*x[1]);
end;

procedure vector_length(out r: Single; constref x: TGVector3f);
begin
  r:=sqrt(x[0]*x[0]
         +x[1]*x[1]
         +x[2]*x[2]);
end;

procedure vector_length(out r: Single; constref x: TGVector4f);
begin
  r:=sqrt(x[0]*x[0]
         +x[1]*x[1]
         +x[2]*x[2]
         +x[3]*x[3]);
end;

function vector_length(x: TGVector2f): Single;
begin
  vector_length(Result,x);
end;

function vector_length(x: TGVector3f): Single;
begin
  vector_length(Result,x);
end;

function vector_length(x: TGVector4f): Single;
begin
  vector_length(Result,x);
end;

procedure vector_normalize(out r: TGVector2f; constref x: TGVector2f);
begin
  r:=x/vector_length(x);
end;

procedure vector_normalize(out r: TGVector3f; constref x: TGVector3f);
begin
  r:=x/vector_length(x);
end;

procedure vector_normalize(out r: TGVector4f; constref x: TGVector4f);
begin
  r:=x/vector_length(x);
end;

function vector_normalize(x: TGVector2f): TGVector2f;
begin
  vector_normalize(Result,x);
end;

function vector_normalize(x: TGVector3f): TGVector3f;
begin
  vector_normalize(Result,x);
end;

function vector_normalize(x: TGVector4f): TGVector4f;
begin
  vector_normalize(Result,x);
end;

procedure vector_copy(out r: TGVector2f; constref x: TGVector2f);
begin
  Move(x,r,SizeOf(TGVector2f));
end;

procedure vector_copy(out r: TGVector3f; constref x: TGVector3f);
begin
  Move(x,r,SizeOf(TGVector3f));
end;

procedure vector_copy(out r: TGVector4f; constref x: TGVector4f);
begin
  Move(x,r,SizeOf(TGVector4f));
end;

procedure vector_copy(out r: TGVector4i; constref x: TGVector4i);
begin
  Move(x,r,SizeOf(TGVector4i));
end;

procedure vector_set(out r: TGVector2f; constref x0, x1: Single);
begin
  r[0]:=x0;
  r[1]:=x1;
end;

procedure vector_set(out r: TGVector3f; constref x0, x1, x2: Single);
begin
  r[0]:=x0;
  r[1]:=x1;
  r[2]:=x2;
end;

procedure vector_set(out r: TGVector4f; constref x0, x1, x2, x3: Single);
begin
  r[0]:=x0;
  r[1]:=x1;
  r[2]:=x2;
  r[3]:=x3;
end;

function vector_set(x0, x1: Single): TGVector2f;
begin
  vector_set(Result,x0,x1);
end;

function vector_set(x0, x1, x2: Single): TGVector3f;
begin
  vector_set(Result,x0,x1,x2);
end;

function vector_set(x0, x1, x2, x3: Single): TGVector4f;
begin
  vector_set(Result,x0,x1,x2,x3);
end;

procedure vector_negative(out r: TGVector2f; constref x: TGVector2f);
begin
  r[0]:=-x[0];
  r[1]:=-x[1];
end;

procedure vector_negative(out r: TGVector3f; constref x: TGVector3f);
begin
  r[0]:=-x[0];
  r[1]:=-x[1];
  r[2]:=-x[2];
end;

procedure vector_negative(out r: TGVector4f; constref x: TGVector4f);
begin
  r[0]:=-x[0];
  r[1]:=-x[1];
  r[2]:=-x[2];
  r[3]:=-x[3];
end;

function vector_negative(x: TGVector2f): TGVector2f;
begin
  vector_negative(Result,x);
end;

function vector_negative(x: TGVector3f): TGVector3f;
begin
  vector_negative(Result,x);
end;

function vector_negative(x: TGVector4f): TGVector4f;
begin
  vector_negative(Result,x);
end;


end.

