unit graphica_globject;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, gl, graphica_vector, graphica_matrix,
  graphica_object, graphica_glrenderinfo;

type

  { TGGLObject }

  TGGLObject = class (TGObjectChilds)
  private

  protected
    FPosition:TGVector3f;
    FScale:TGVector3f;
    FSize:TGVector3f;
    FRotate:TGVector3f;
    FNormal:TGVector4f;
    FSizeMatrix:TGMatrix4x4f;
    FModelMatrix:TGMatrix4x4f;
    FAbsoluteModelMatrix4:TGMatrix4x4f;
    FTAMM:TGMatrix4x4f;

    FVisible:Boolean;
    FSelected:Boolean;
    FFinishRenderFlag:Boolean;

    procedure DoRender(const renderinfo:TGRenderInfo);dynamic;
    procedure FinishRender(const renderinfo:TGRenderInfo);dynamic;
    procedure InitObject;dynamic;
  public
    constructor Create();
    constructor Create(own:TGGLObject);

    procedure Render(const renderinfo:TGRenderInfo);

    procedure updateMatrix();dynamic;

    procedure SetPosition(constref x,y,z:Single);
    procedure SetPosition(constref v:TGVector3f);
    procedure SetPosition(constref v:PGVector3f);
    function GetPosition():TGVector3f;inline;
    function GetAbsolutePosition():TGVector3f;

    procedure SetSize(constref x,y,z:Single);
    procedure SetSize(constref v:TGVector3f);
    function GetSize():TGVector3f;inline;

    procedure SetScale(constref x,y,z:Single);
    procedure SetScale(constref v:TGVector3f);
    function GetScale():TGVector3f;inline;

    procedure SetRotate(constref x,y,z:Single);
    procedure SetRotate(constref v:TGVector3f);
    function GetRotate():TGVector3f;inline;


    function GetNormal():TGVector3f;inline;


    property Visible:Boolean read FVisible write FVisible;
    property Selected:Boolean read FSelected write FSelected;
    property SizeMatrix:TGMatrix4x4f read FSizeMatrix;
    property ModelMatrix:TGMatrix4x4f read FModelMatrix;
    property AbsoluteModelMatrix4:TGMatrix4x4f read FAbsoluteModelMatrix4;
    property TAMM:TGMatrix4x4f read FTAMM;
  end;



implementation
uses graphica_2d3d;

{ TGGLObject }

procedure TGGLObject.InitObject;
begin
  vector_copy(FPosition,vector3f_zero);
  vector_copy(FScale,vector3f_one);
  vector_copy(FSize,vector3f_one);
  vector_copy(FRotate,vector3f_zero);
  matrix_copy(FSizeMatrix,matrix4x4f_One);
  matrix_copy(FAbsoluteModelMatrix4,matrix4x4f_One);
  updateMatrix();
  FVisible:=true;
  FSelected:=true;
  FFinishRenderFlag:=false;
end;

procedure TGGLObject.DoRender(const renderinfo: TGRenderInfo);
begin

end;

procedure TGGLObject.FinishRender(const renderinfo: TGRenderInfo);
begin

end;

constructor TGGLObject.Create;
begin
  inherited Create();
  InitObject;
end;

constructor TGGLObject.Create(own: TGGLObject);
begin
  inherited Create(own);
  InitObject;
end;

procedure TGGLObject.Render(const renderinfo: TGRenderInfo);
var
  i:Integer;
  obj:TGGLObject;
begin
  if (not FVisible) then Exit;
//  if (select<>nil) and not FSelected then Exit;
  DoRender(renderinfo);
  for i:=0 to FChilds.Count-1 do begin
    obj:=TGGLObject(FChilds[i]);
    obj.Render(renderinfo);
  end;
  if FFinishRenderFlag then
    FinishRender(renderinfo);
end;

procedure TGGLObject.updateMatrix;
var
  TranslateMatrix,RotateMatrix,ScaleMatrix,tmp:TGMatrix4x4f;
  i:Integer;
  obj:TGGLObject;
begin
  matrix_transformation(TranslateMatrix,FPosition);;
  matrix_scale(ScaleMatrix,FScale);
  matrix_copy(RotateMatrix,matrix4x4f_one);
//  matrix_rotatrion(RotateMatrix,FRotate);
  matrix_scale(FSizeMatrix,FSize);
  matrix_mul(tmp,TranslateMatrix,RotateMatrix);
  matrix_mul(FModelMatrix,tmp,ScaleMatrix);

  if FOnwer=nil then
    matrix_copy(FAbsoluteModelMatrix4,FModelMatrix)
  else
    matrix_mul(FAbsoluteModelMatrix4,TGGLObject(FOnwer).FAbsoluteModelMatrix4,
                FModelMatrix);
  matrix_transopse(FTAMM,FAbsoluteModelMatrix4);

  matrix_mul(FNormal,AbsoluteModelMatrix4,vector4f_normal_plane_xy);

  for i:=0 to FChilds.Count-1 do begin
    obj:=TGGLObject(FChilds[i]);
    obj.updateMatrix();
  end;
end;

procedure TGGLObject.SetPosition(constref x, y, z: Single);
begin
  vector_set(FPosition,x,y,z);
  updateMatrix();
end;

procedure TGGLObject.SetPosition(constref v: TGVector3f);
begin
  vector_copy(FPosition,v);
  updateMatrix();
end;

procedure TGGLObject.SetPosition(constref v: PGVector3f);
begin
  vector_copy(FPosition,v^);
  updateMatrix();
end;

function TGGLObject.GetPosition: TGVector3f;
begin
  Result:=FPosition;
end;

function TGGLObject.GetAbsolutePosition: TGVector3f;
begin
  vector_transformation(Result,FAbsoluteModelMatrix4);
end;

procedure TGGLObject.SetSize(constref x, y, z: Single);
begin
  vector_set(FSize,x,y,z);
  updateMatrix();
end;

procedure TGGLObject.SetSize(constref v: TGVector3f);
begin
  vector_copy(FSize,v);
  updateMatrix();
end;

function TGGLObject.GetSize: TGVector3f;
begin
  vector_scale(Result,FSizeMatrix);
end;

procedure TGGLObject.SetScale(constref x, y, z: Single);
begin
  vector_set(FScale,x,y,z);
  updateMatrix();
end;

procedure TGGLObject.SetScale(constref v: TGVector3f);
begin
  vector_copy(FSize,v);
  updateMatrix();
end;

function TGGLObject.GetScale: TGVector3f;
begin
  Result:=FScale;
end;

procedure TGGLObject.SetRotate(constref x, y, z: Single);
begin

end;

procedure TGGLObject.SetRotate(constref v: TGVector3f);
begin

end;

function TGGLObject.GetRotate: TGVector3f;
begin

end;

function TGGLObject.GetNormal: TGVector3f;
begin
  Result:=FNormal;
end;

end.

