unit graphica_glselect;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, gl, graphica_vector,graphica_matrix, graphica_object,
  graphica_glview, graphica_2d3d, graphica_glshader;

type

  { TGLSelect }

  TGLSelect = class
  protected
    FMouseX, FMouseY: integer;
  public
    procedure setParameters(constref x, y:integer); dynamic;
    procedure Start(constref MVPMatrix, ModelMatrix, SizeMatrix: TGMatrix4x4f);
      dynamic; abstract;
    procedure Stop(const obj: TGObject;const view: TGGLView); dynamic; abstract;
    procedure Clear();virtual;abstract;
  end;


  { TGLSelectColorList }

  TGLSelectColorList = class (TGLSelect)
  private
    FColorCount:DWord;
    FMVP,FMA,FMS,FColorID:GLint;
    FObjectList:TFPList;
    FShader:TGGLShader;

    function getCount():Integer;

  public
   constructor Create();
   destructor Destroy();override;

   procedure setParameters(constref x, y: Integer);override;
   procedure Start(constref MVPMatrix,ModelMatrix,
              SizeMatrix: TGMatrix4x4f);override;
   procedure Stop(const obj: TGObject;const view: TGGLView);override;
   procedure Clear();override;

   function getObject(index:Integer):TGObject;inline;
   function findObject(const obj: TGObject):Integer;inline;

   property Shader:TGGLShader read FShader;
   property Count:Integer read getCount;

//   property ObjectList:TFPList read FObjectList;
  end;


  function getSelectColor(x, y: Integer; viewport:PGVector4i):TGVector4b;inline;
  procedure getSelectColor(x, y: Integer; viewport:PGVector4i;color:PGVector4b);//inline;


implementation
uses GLext;


{$i graphica_glselect_shader.inc}

function getSelectColor(x, y: Integer; viewport: PGVector4i): TGVector4b;
begin
  glReadPixels( x, viewport^[3]-y-1,1, 1, GL_RGBA, GL_UNSIGNED_BYTE, @Result);
end;

procedure getSelectColor(x, y: Integer; viewport: PGVector4i; color:PGVector4b);
begin
  glReadPixels( x, viewport^[3]-y-1,1, 1, GL_RGBA, GL_UNSIGNED_BYTE, color);
end;

{ TGLSelectColorList }

function TGLSelectColorList.getCount: Integer;
begin
  Result:=FObjectList.Count;
end;

constructor TGLSelectColorList.Create;
begin
  FColorCount:=$FF000000;//Альфа = 255
  FObjectList:=TFPList.Create;
  FShader:=TGGLShader.Create(selectVColor,selectFColor);
  FMVP:=glGetUniformLocation(FShader.ID,'MVP');
  FMS:=glGetUniformLocation(FShader.ID,'MMS');
  FMA:=glGetUniformLocation(FShader.ID,'MA');
  FColorID:=glGetAttribLocation(FShader.ID,'Color');
end;

destructor TGLSelectColorList.Destroy;
begin
  Clear;
  FreeAndNil(FObjectList);
  FreeAndNil(FShader);
  inherited Destroy;
end;

procedure TGLSelectColorList.setParameters(constref x, y: Integer);
begin
  inherited setParameters(x, y);
  Clear;

end;

procedure TGLSelectColorList.Start(constref MVPMatrix, ModelMatrix,
  SizeMatrix: TGMatrix4x4f);
var
  FColorCount4f:TGVector4f;
  c:PByteArray;
begin
  Inc(FColorCount);
  c:=@FColorCount;
  FColorCount4f[0]:=c^[0]/255;
  FColorCount4f[1]:=c^[1]/255;
  FColorCount4f[2]:=c^[2]/255;
  FColorCount4f[3]:=255;

  FShader.Active();
  glUniformMatrix4fv(FMVP, 1, GL_FALSE, PGLfloat(@MVPMatrix));
  glUniformMatrix4fv(FMA, 1, GL_FALSE, PGLfloat(@ModelMatrix));
  glUniformMatrix4fv(FMS, 1, GL_FALSE, PGLfloat(@SizeMatrix));
  glVertexAttrib4fv(FColorID,@FColorCount4f);
end;

procedure TGLSelectColorList.Stop(const obj: TGObject; const view: TGGLView);
var
  colorInt:DWord;
begin
  FShader.Deactive();
  getSelectColor(FMouseX,FMouseY,@view.ViewPort,@colorInt);
  if(FColorCount=colorInt) then
     FObjectList.Add(obj);
end;

procedure TGLSelectColorList.Clear;
begin
  FObjectList.Clear;
  FColorCount:=$FF000000;//Alpha=255;
end;

function TGLSelectColorList.getObject(index: Integer): TGObject;
begin
  Result:=TGObject(FObjectList.Items[index]);
end;

function TGLSelectColorList.findObject(const obj: TGObject): Integer;
begin
  Result:=FObjectList.IndexOf(obj);
end;

{ TGLSelect }

procedure TGLSelect.setParameters(constref x, y: integer);
begin
  FMouseX:=x;
  FMouseY:=y;
end;

end.

