unit graphica_glshader;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, gl;

type

  { TGGLShader }

  TGGLShader = class
  private
    FProgramID:GLint;
    FCompiled:Boolean;
  public
    procedure Active();inline;
    procedure Deactive();inline;
    constructor Create(const src_vertex,src_fragment:string);
    destructor Destroy(); override;
    property ID:GLint read FProgramID;
    property Compiled:Boolean read FCompiled;
  end;

implementation
uses GLext;


function _loadshader(_type: GLenum; src:pchar): GLint;
var
  compiled,infoLen:GLint;
  infoLog:string;
begin
  Result:=glCreateShader(_type);
  glShaderSource(Result,1,@src,nil);
  glCompileShader(Result);
  glGetShaderiv(Result, GL_COMPILE_STATUS, @compiled);
  if(compiled<>GL_TRUE) then begin
      glGetShaderiv(Result, GL_INFO_LOG_LENGTH, @infoLen);
      if infoLen>0 then begin
         SetLength(infoLog,infoLen);
         glGetShaderInfoLog(Result, infoLen, nil, @infoLog[1]);
         writeln(infoLog);
      end;
      glDeleteShader(Result);
      Result:=-1;
  end;
end;

function _linkshader(vertexShader,fragmentShader:GLuint;ProgramID:GLint): boolean;
var
   linked,infoLen:GLint;
   infoLog:string;
begin
  Result:=true;
  glAttachShader(ProgramID, vertexShader);
  glAttachShader(ProgramID, fragmentShader);
  glLinkProgram(ProgramID);
  glGetProgramiv(ProgramID, GL_LINK_STATUS, @linked);
  if(linked<>GL_TRUE) then begin
      glGetShaderiv(ProgramID, GL_INFO_LOG_LENGTH, @infoLen);
      if infoLen>0 then begin
         SetLength(infoLog,infoLen);
         glGetShaderInfoLog(ProgramID, infoLen, nil, @infoLog[1]);
         writeln(infoLog);
      end;
      glDeleteProgram(ProgramID);
      Result:=false;
  end;
end;



{ TGGLShader }
procedure TGGLShader.Active;
begin
  glUseProgram(FProgramID);
end;

procedure TGGLShader.Deactive;
begin
  glUseProgram(0);
end;

constructor TGGLShader.Create(const src_vertex, src_fragment: string);
var
   vertexShader,fragmentShader:GLuint;
begin
  FCompiled:=False;
  FProgramID:=glCreateProgram();
  vertexShader:=_loadshader(GL_VERTEX_SHADER,pchar(src_vertex));
  if(vertexShader=-1) then begin
    glDeleteProgram(FProgramID);
    exit;
  end;
  fragmentShader:=_loadshader(GL_FRAGMENT_SHADER,pchar(src_fragment));
  if(fragmentShader=-1) then begin
    glDeleteShader(vertexShader);
    glDeleteProgram(FProgramID);
    exit;
  end;

  if not _linkshader(vertexShader,fragmentShader,FProgramID) then begin
    glDeleteShader(vertexShader);
    glDeleteShader(fragmentShader);
    glDeleteProgram(FProgramID);
    exit;
  end;
  glDeleteShader(vertexShader);
  glDeleteShader(fragmentShader);
  FCompiled:=true;
end;

destructor TGGLShader.Destroy;
begin
  glDeleteProgram(FProgramID);
  inherited Destroy;
end;

initialization
  Load_GL_VERSION_3_2();

end.

