unit graphica_gltexture;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, gl;

type

  { TGGLTexture }

  TGGLTexture = class
  private
    FFlagUpdate:Integer;
    function checkUpdate:Boolean;
  protected
    FTextureID:GLint;

  public
     constructor Create();
     destructor Destroy();override;

     procedure bind();inline;
     procedure unBind();inline;


     procedure load(const width,height:Integer; const internalformat,format
        :GLEnum; const buf:Pointer);dynamic;
     procedure parameter(const param_filter_mag,param_filter_min,
        wap_s,wap_t, env_mode: GLint);inline;
     property ID:TGLint read FTextureID;
     property FlagUpdate:Boolean read checkUpdate;
  end;
implementation

{ TGGLTexture }

function TGGLTexture.checkUpdate: Boolean;
var
  flag:Integer;
begin
  flag:=1;
  InterlockedCompareExchange(flag,0,FFlagUpdate);
  Result:=flag=0;
end;

constructor TGGLTexture.Create;
begin
  FFlagUpdate:=0;
  glGenTextures(1, @FTextureID);
end;

destructor TGGLTexture.Destroy;
begin
  glDeleteTextures(1,@FTextureID);
  inherited Destroy;
end;

procedure TGGLTexture.bind;
begin
  glBindTexture(GL_TEXTURE_2D,FTextureID);
end;

procedure TGGLTexture.unBind;
begin
  glBindTexture(GL_TEXTURE_2D,0);
end;

procedure TGGLTexture.load(const width, height: Integer; const internalformat,
  format: GLEnum; const buf: Pointer);
var
  flag:Integer;
begin
  glTexImage2D(GL_TEXTURE_2D, 0, internalformat, width, height, 0,
      format, GL_UNSIGNED_BYTE, buf);
  InterlockedCompareExchange(FFlagUpdate,1,0);
end;

procedure TGGLTexture.parameter(const param_filter_mag, param_filter_min,
  wap_s, wap_t, env_mode: GLint);
begin
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, param_filter_mag);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, param_filter_min);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
end;

end.

