unit graphica_glview;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, graphica_vector, graphica_matrix;

type

  TGGLViewRay =record
    pointZNear:TGVector4f;
    pointZFar:TGVector4f;
    normal:TGVector4f;
  end;
  PGGLViewRay=^TGGLViewRay;


  { TGGLView }

  TGGLView = class
  private
    FPojection:TGMatrix4x4f;
    FView:TGMatrix4x4f;
    FMVP:TGMatrix4x4f;
    FGLMVP:TGMatrix4x4f;
    FMVPInverse:TGMatrix4x4f;
    FViewPort:TGVector4i;
    FBackgroundColor:TGVector4f;
    procedure updateMVP();
  public
    constructor Create();
    procedure atLook(const posX,posY,posZ,targetX,
          targetY,targetZ,upX,upY,upZ:single);
    procedure setOrtho(const left,right,top,bottom,znear,zfar:single);
    procedure setOrtho2d(const width,height,znear,zfar:single);
    procedure setViewPort(const x,y,w,h:Integer);
    procedure setBackgroundColor(const r,g,b,a:Single);



    procedure drawGLViewport();inline;
    procedure drawGLClear();inline;
    procedure drawGLParams();inline;

    function getViewRay(constref x,y:Single):TGGLViewRay;

    function getEye():TGVector4f;
    function getTarger():TGVector3f;
    function getUp():TGVector3f;

    property MVP:TGMatrix4x4f read FMVP;
    property GLMVP:TGMatrix4x4f read FGLMVP;
    property MVPInverse:TGMatrix4x4f read FMVPInverse;
    property View:TGMatrix4x4f read FView;
    property Pojection:TGMatrix4x4f read FPojection;
    property ViewPort:TGVector4i read FViewPort;
    property BackgroundColor:TGVector4f read FBackgroundColor;
  end;

implementation
uses gl, graphica_2d3d;
{ TGGLView }

procedure TGGLView.updateMVP;
begin
  matrix_mul(FMVP,FPojection,FView);
  matrix_inverse(FMVPInverse,FMVP);
  matrix_transopse(FGLMVP,FMVP);
end;

constructor TGGLView.Create;
begin
  matrix_copy(FPojection,matrix4x4f_one);
  matrix_copy(FView,matrix4x4f_one);
  matrix_copy(FMVP,matrix4x4f_one);
  vector_set(FBackgroundColor,0,0,0,1);
end;

procedure TGGLView.atLook(const posX, posY, posZ, targetX, targetY, targetZ,
  upX, upY, upZ: single);
begin
  create_matrix_view(FView,posX, posY, posZ, targetX, targetY, targetZ,
      upX, upY, upZ);
  updateMVP();
end;

procedure TGGLView.setOrtho(const left, right, top, bottom, znear, zfar: single
  );
begin
  create_matrix_projection_ortho(FPojection,left, right, top, bottom, znear,
       zfar);
  updateMVP();
end;

procedure TGGLView.setOrtho2d(const width, height, znear, zfar: single);
begin
  setOrtho(-width/2,width/2,-height/2,height/2,znear,zfar);
end;

procedure TGGLView.setViewPort(const x, y, w, h: Integer);
begin
  FViewPort[0]:=x;FViewPort[1]:=y;FViewPort[2]:=w;FViewPort[3]:=h;
end;

procedure TGGLView.setBackgroundColor(const r, g, b, a: Single);
begin
  FBackgroundColor[0]:=r;
  FBackgroundColor[1]:=g;
  FBackgroundColor[2]:=b;
  FBackgroundColor[3]:=a;
end;

procedure TGGLView.drawGLViewport;
begin
  glViewport(FViewPort[0],FViewPort[1],
             FViewPort[2],FViewPort[3]);
end;

procedure TGGLView.drawGLClear;
begin
   glClearColor(FBackgroundColor[0],FBackgroundColor[1],FBackgroundColor[2],
               FBackgroundColor[3]);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glClearDepth(1);
  glDepthMask( 1 );
end;

procedure TGGLView.drawGLParams;
begin
  glEnable(GL_BLEND);
  glEnable( GL_DEPTH_TEST );
  glEnable(GL_ALPHA_TEST);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glAlphaFunc(GL_GREATER, 0.01);
  glEnable(GL_TEXTURE_2D);
end;

function TGGLView.getViewRay(constref x, y: Single): TGGLViewRay;
var
   _x,_y:Single;
   _zNear,_zFar:TGVector4f;
begin
  _x:=(2*x)/FViewPort[2]-1;
  _y:=1-(2*y)/FViewPort[3];
  _zNear:=vector_set(_x,_y,-1,1);
  _zFar:=vector_set(_x,_y,1,1);
  matrix_mul(Result.pointZNear,MVPInverse,_zNear);
  matrix_mul(Result.pointZFar,MVPInverse,_zFar);
  Result.pointZNear:=Result.pointZNear/Result.pointZNear[3];
  Result.pointZFar:=Result.pointZFar/Result.pointZFar[3];
  Result.normal:=Result.pointZFar-Result.pointZNear;
  Result.normal:=vector_normalize(Result.normal);
end;

function TGGLView.getEye: TGVector4f;
begin
  vector_transformation(Result,FView);
end;

function TGGLView.getTarger: TGVector3f;
begin

end;

function TGGLView.getUp: TGVector3f;
begin

end;

end.

